# Generated by Django 2.1.4 on 2018-12-16 22:24

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0055_auto_20181022_1937'),
    ]

    operations = [
        migrations.AlterField(
            model_name='call',
            name='last_modified_by',
            field=models.ForeignKey(blank=True, limit_choices_to={'is_staff': True}, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_calllstmodified', to=settings.AUTH_USER_MODEL, verbose_name='Last modified by'),
        ),
        migrations.AlterField(
            model_name='call',
            name='staff',
            field=models.ForeignKey(blank=True, limit_choices_to={'is_staff': True}, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_relcallstaff', to=settings.AUTH_USER_MODEL, verbose_name='Staff'),
        ),
        migrations.AlterField(
            model_name='contact',
            name='last_modified_by',
            field=models.ForeignKey(blank=True, limit_choices_to={'is_staff': True}, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Last modified by'),
        ),
        migrations.AlterField(
            model_name='contract',
            name='last_modified_by',
            field=models.ForeignKey(limit_choices_to={'is_staff': True}, on_delete=django.db.models.deletion.CASCADE, related_name='db_contractlstmodified', to=settings.AUTH_USER_MODEL, verbose_name='Last modified by'),
        ),
        migrations.AlterField(
            model_name='contract',
            name='staff',
            field=models.ForeignKey(blank=True, limit_choices_to={'is_staff': True}, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_relcontractstaff', to=settings.AUTH_USER_MODEL, verbose_name='Staff'),
        ),
        migrations.AlterField(
            model_name='genericprojectlink',
            name='last_modified_by',
            field=models.ForeignKey(limit_choices_to={'is_staff': True}, on_delete=django.db.models.deletion.CASCADE, related_name='db_project_link_last_modified', to=settings.AUTH_USER_MODEL, verbose_name='Last modified by'),
        ),
        migrations.AlterField(
            model_name='generictasklink',
            name='last_modified_by',
            field=models.ForeignKey(limit_choices_to={'is_staff': True}, on_delete=django.db.models.deletion.CASCADE, related_name='db_task_link_last_modified', to=settings.AUTH_USER_MODEL, verbose_name='Last modified by'),
        ),
        migrations.AlterField(
            model_name='position',
            name='product_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.ProductType', verbose_name='Product'),
        ),
        migrations.AlterField(
            model_name='producttype',
            name='last_modified_by',
            field=models.ForeignKey(blank=True, limit_choices_to={'is_staff': True}, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Last modified by'),
        ),
        migrations.AlterField(
            model_name='project',
            name='last_modified_by',
            field=models.ForeignKey(limit_choices_to={'is_staff': True}, on_delete=django.db.models.deletion.CASCADE, related_name='db_project_last_modified', to=settings.AUTH_USER_MODEL, verbose_name='Last modified by'),
        ),
        migrations.AlterField(
            model_name='project',
            name='project_manager',
            field=models.ForeignKey(blank=True, limit_choices_to={'is_staff': True}, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_rel_project_staff', to=settings.AUTH_USER_MODEL, verbose_name='Staff'),
        ),
        migrations.AlterField(
            model_name='salesdocument',
            name='last_modified_by',
            field=models.ForeignKey(blank='True', limit_choices_to={'is_staff': True}, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_lstscmodified', to=settings.AUTH_USER_MODEL, verbose_name='Last modified by'),
        ),
        migrations.AlterField(
            model_name='salesdocument',
            name='staff',
            field=models.ForeignKey(blank=True, limit_choices_to={'is_staff': True}, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='db_relscstaff', to=settings.AUTH_USER_MODEL, verbose_name='Staff'),
        ),
    ]



import datetime
import pytest
from django.test import TestCase
from koalixcrm.crm.factories.factory_user import AdminUserFactory
from koalixcrm.crm.factories.factory_customer_billing_cycle import StandardCustomerBillingCycleFactory
from koalixcrm.crm.factories.factory_customer import StandardCustomerFactory
from koalixcrm.crm.factories.factory_customer_group import StandardCustomerGroupFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.crm.factories.factory_reporting_period import StandardReportingPeriodFactory
from koalixcrm.crm.factories.factory_human_resource import StandardHumanResourceFactory
from koalixcrm.crm.factories.factory_work import StandardWorkFactory
from koalixcrm.crm.factories.factory_task import StandardTaskFactory
from koalixcrm.crm.factories.factory_reporting_period_status import DoneReportingPeriodStatusFactory
from koalixcrm.crm.factories.factory_estimation import StandardHumanResourceEstimationToTaskFactory
from koalixcrm.global_support_functions import make_date_utc
from koalixcrm.crm.exceptions import ReportingPeriodDoneDeleteNotPossible


class TestWorkDelete(TestCase):
    def setUp(self):
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        start_date = (datetime_now - datetime.timedelta(days=30)).date()
        end_date_first_task = (datetime_now + datetime.timedelta(days=30)).date()
        end_date_second_task = (datetime_now + datetime.timedelta(days=60)).date()

        self.test_billing_cycle = StandardCustomerBillingCycleFactory.create()
        self.test_user = AdminUserFactory.create()
        self.test_customer_group = StandardCustomerGroupFactory.create()
        self.test_customer = StandardCustomerFactory.create(is_member_of=(self.test_customer_group,))
        self.test_currency = StandardCurrencyFactory.create()
        self.human_resource = StandardHumanResourceFactory.create()
        self.test_reporting_period = StandardReportingPeriodFactory.create()
        self.test_1st_task = StandardTaskFactory.create(title="1st Test Task",
                                                        project=self.test_reporting_period.project)
        self.estimation_1st_task = StandardHumanResourceEstimationToTaskFactory(task=self.test_1st_task,
                                                                                date_from=start_date,
                                                                                date_until=end_date_first_task)
        self.test_2nd_task = StandardTaskFactory.create(title="2nd Test Task",
                                                        project=self.test_reporting_period.project)
        self.estimation_2nd_task = StandardHumanResourceEstimationToTaskFactory(task=self.test_2nd_task,
                                                                                date_from=start_date,
                                                                                date_until=end_date_second_task)

    @pytest.mark.back_end_tests
    def test_work_delete(self):
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        datetime_later_1 = make_date_utc(datetime.datetime(2024, 1, 1, 2, 00))
        datetime_later_2 = make_date_utc(datetime.datetime(2024, 1, 1, 3, 30))
        self.datetime_later_3 = make_date_utc(datetime.datetime(2024, 1, 1, 5, 45))
        datetime_later_4 = make_date_utc(datetime.datetime(2024, 1, 1, 6, 15))
        date_now = datetime_now.date()
        self.assertEqual(
            (self.test_1st_task.planned_duration()).__str__(), "60")
        self.assertEqual(
            (self.test_1st_task.planned_costs()).__str__(), "0.00")
        self.assertEqual(
            (self.test_2nd_task.planned_duration()).__str__(), "90")
        self.assertEqual(
            (self.test_2nd_task.planned_costs()).__str__(), "0.00")
        StandardWorkFactory.create(
            human_resource=self.human_resource,
            date=date_now,
            start_time=datetime_now,
            stop_time=datetime_later_1,
            task=self.test_1st_task,
            reporting_period=self.test_reporting_period
        )
        StandardWorkFactory.create(
            human_resource=self.human_resource,
            date=date_now,
            start_time=datetime_later_1,
            stop_time=datetime_later_2,
            task=self.test_1st_task,
            reporting_period=self.test_reporting_period
        )
        work_cannot_be_deleted = StandardWorkFactory.create(
            human_resource=self.human_resource,
            date=date_now,
            start_time=datetime_now,
            stop_time=self.datetime_later_3,
            task=self.test_2nd_task,
            reporting_period=self.test_reporting_period
        )
        work_can_be_deleted = StandardWorkFactory.create(
            human_resource=self.human_resource,
            date=date_now,
            start_time=datetime_now,
            stop_time=datetime_later_4,
            task=self.test_2nd_task,
            reporting_period=self.test_reporting_period
        )
        self.assertEqual(
            (self.test_1st_task.effective_effort(reporting_period=None)).__str__(), "3.5")
        self.assertEqual(
            (self.test_2nd_task.effective_effort(reporting_period=None)).__str__(), "12")
        work_can_be_deleted.delete()
        self.assertEqual(
            (self.test_1st_task.effective_effort(reporting_period=None)).__str__(), "3.5")
        self.assertEqual(
            (self.test_2nd_task.effective_effort(reporting_period=None)).__str__(), "5.75")
        status_done = DoneReportingPeriodStatusFactory.create()
        self.test_reporting_period.status = status_done
        with pytest.raises(ReportingPeriodDoneDeleteNotPossible):
            work_cannot_be_deleted.delete()
        with pytest.raises(ReportingPeriodDoneDeleteNotPossible):
            work_cannot_be_deleted.stop_time = self.datetime_later_3
            work_cannot_be_deleted.save()




from rest_framework import serializers
from django.contrib.auth.models import User


class UserSerializer(serializers.HyperlinkedModelSerializer):
    email = serializers.EmailField()
    username = serializers.CharField(max_length=100)

    class Meta:
        model = User
        fields = ('username',
                  'email',
                  'id')



# -*- coding: utf-8 -*-

import factory
from koalixcrm.djangoUserExtension.models import *
from filebrowser.base import FileObject


class StandardQuoteTemplateFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = QuoteTemplate

    title = "This is a test Quote Template"
    xsl_file = FileObject("~/path/to/xsl_file.xsl")
    fop_config_file = FileObject("~/path/to/fop_config_file.xml")
    logo = FileObject("~/path/to/logo_file.jpg")


class StandardInvoiceTemplateFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = InvoiceTemplate

    title = "This is a test Invoice Template"
    xsl_file = FileObject("~/path/to/xsl_file.xsl")
    fop_config_file = FileObject("~/path/to/fop_config_file.xml")
    logo = FileObject("~/path/to/logo_file.jpg")


class StandardDeliveryNoteTemplateFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = DeliveryNoteTemplate

    title = "This is a test Delivery Note Template"
    xsl_file = FileObject("~/path/to/xsl_file.xsl")
    fop_config_file = FileObject("~/path/to/fop_config_file.xml")
    logo = FileObject("~/path/to/logo_file.jpg")


class StandardPaymentReminderTemplateFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = PaymentReminderTemplate

    title = "This is a test Payment Reminder Template"
    xsl_file = FileObject("~/path/to/xsl_file.xsl")
    fop_config_file = FileObject("~/path/to/fop_config_file.xml")
    logo = FileObject("~/path/to/logo_file.jpg")


class StandardPurchaseOrderTemplateFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = PurchaseOrderTemplate

    title = "This is a test Purchase Order Template"
    xsl_file = FileObject("~/path/to/xsl_file.xsl")
    fop_config_file = FileObject("~/path/to/fop_config_file.xml")
    logo = FileObject("~/path/to/logo_file.jpg")


class StandardPurchaseConfirmationTemplateFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = PurchaseConfirmationTemplate

    title = "This is a test Purchase Confirmation Template"
    xsl_file = FileObject("~/path/to/xsl_file.xsl")
    fop_config_file = FileObject("~/path/to/fop_config_file.xml")
    logo = FileObject("~/path/to/logo_file.jpg")


class StandardBalanceSheetTemplateFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = BalanceSheetTemplate

    title = "This is a test Balance Sheet Template"
    xsl_file = FileObject("~/path/to/xsl_file.xsl")
    fop_config_file = FileObject("~/path/to/fop_config_file.xml")
    logo = FileObject("~/path/to/logo_file.jpg")


class StandardProfitLossStatementTemplateFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ProfitLossStatementTemplate

    title = "This is a test Profit Loss Statement Template"
    xsl_file = FileObject("~/path/to/xsl_file.xsl")
    fop_config_file = FileObject("~/path/to/fop_config_file.xml")
    logo = FileObject("~/path/to/logo_file.jpg")


class StandardMonthlyProjectSummaryTemplateFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = MonthlyProjectSummaryTemplate

    title = "This is a test Purchase Order Template"
    xsl_file = FileObject("~/path/to/xsl_file.xsl")
    fop_config_file = FileObject("~/path/to/fop_config_file.xml")
    logo = FileObject("~/path/to/logo_file.jpg")



# -*- coding: utf-8 -*-

from django.http import HttpResponseRedirect, Http404
from django.shortcuts import render
from django.template.context_processors import csrf
from django.contrib.admin import helpers
from django.contrib.admin.widgets import *
from koalixcrm.djangoUserExtension.exceptions import TooManyUserExtensionsAvailable


class ReportingPeriodMissingForm(forms.Form):
    NEXT_STEPS = (
        ('add_reporting_period', 'Add Reporting Period'),
        ('return_to_start', 'Return To Start'),
    )
    next_steps = forms.ChoiceField(required=True,
                                   widget=forms.Select,
                                   choices=NEXT_STEPS)


def reporting_period_missing(request):
    try:
        if request.POST.get('post'):
            if 'confirm_selection' in request.POST:
                reporting_period_missing_form = ReportingPeriodMissingForm(request.POST)
                if reporting_period_missing_form.is_valid():
                    if reporting_period_missing_form.cleaned_data['next_steps'] == 'return_to_start':
                        return HttpResponseRedirect('/admin/')
                    else:
                        return HttpResponseRedirect('/admin/crm/reportingperiod/add/')
        else:
            reporting_period_missing_form = ReportingPeriodMissingForm(initial={'next_steps': 'create_user_extension'})
        title = "User Extension Missing"
        description = "The operation you have selected requires an existing 'Reporting Period' element to exist" \
                      "for the project. For one of the required project this 'Reporting Period' did not exist" \
                      "Please choose one of the available options and proceed with the intended " \
                      "operation afterwards"
        c = {'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME,
             'form': reporting_period_missing_form,
             'description': description,
             'title': title}
        c.update(csrf(request))
        return render(request, 'crm/admin/exception.html', c)
    except TooManyUserExtensionsAvailable:
        raise Http404





# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import PurchaseOrder
from koalixcrm.crm.factories.factory_supplier import StandardSupplierFactory
from koalixcrm.crm.factories.factory_sales_document import StandardSalesDocumentFactory


class StandardPurchaseOrderFactory(StandardSalesDocumentFactory):
    class Meta:
        model = PurchaseOrder

    supplier = factory.SubFactory(StandardSupplierFactory)
    status = "C"



import pytest
import datetime
from django.test import TestCase
from koalixcrm.crm.documents.calculations import Calculations
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.crm.factories.factory_quote import StandardQuoteFactory
from koalixcrm.crm.factories.factory_sales_document_position import StandardSalesDocumentPositionFactory
from koalixcrm.crm.factories.factory_product_type import StandardProductTypeFactory
from koalixcrm.crm.factories.factory_product_price import StandardPriceFactory
from koalixcrm.crm.factories.factory_customer import StandardCustomerFactory
from koalixcrm.crm.factories.factory_customer_group import StandardCustomerGroupFactory, AdvancedCustomerGroupFactory
from koalixcrm.crm.factories.factory_tax import StandardTaxFactory
from koalixcrm.crm.factories.factory_unit import StandardUnitFactory, SmallUnitFactory
from koalixcrm.crm.factories.factory_customer_group_transform import StandardCustomerGroupTransformFactory
from koalixcrm.crm.factories.factory_unit_transform import StandardUnitTransformFactory
from koalixcrm.crm.factories.factory_currency_transform import StandardCurrencyTransformFactory
from koalixcrm.global_support_functions import make_date_utc


class DocumentCalculationsTest(TestCase):
    def setUp(self):
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        start_date = (datetime_now - datetime.timedelta(days=30)).date()
        end_date = (datetime_now + datetime.timedelta(days=30)).date()
        self.tax = StandardTaxFactory.create(tax_rate=10)
        self.test_currency_with_rounding = StandardCurrencyFactory.create(rounding=1)
        self.test_currency_without_rounding = StandardCurrencyFactory.create(
            rounding=None,
            description='Euro',
            short_name='EUR'
        )
        self.alternative_currency = self.test_currency_without_rounding
        self.customer_group = StandardCustomerGroupFactory.create()
        self.alternative_customer_group = AdvancedCustomerGroupFactory.create()
        self.customer = StandardCustomerFactory.create()
        self.customer.is_member_of.add(self.customer_group)
        self.customer.save()
        self.unit = StandardUnitFactory.create()
        self.alternative_unit = SmallUnitFactory.create()
        self.product_without_dates = StandardProductTypeFactory.create(
            product_type_identifier="A",
            tax=self.tax
        )
        self.product_without_date_from = StandardProductTypeFactory.create(
            product_type_identifier="B",
            tax=self.tax
        )
        self.product_without_date_until = StandardProductTypeFactory.create(
            product_type_identifier="C",
            tax=self.tax
        )
        self.product_without_customer_group = StandardProductTypeFactory.create(
            product_type_identifier="D",
            tax=self.tax
        )
        self.product_with_currency_rounding = StandardProductTypeFactory.create(
            product_type_identifier="E",
            tax=self.tax
        )
        self.product_without_currency_rounding = StandardProductTypeFactory.create(
            product_type_identifier="F",
            tax=self.tax
        )
        self.product_with_alternative_customer_group = StandardProductTypeFactory.create(
            product_type_identifier="G",
            tax=self.tax
        )
        self.product_with_alternative_unit = StandardProductTypeFactory.create(
            product_type_identifier="H",
            tax=self.tax
        )
        self.product_with_alternative_currency = StandardProductTypeFactory.create(
            product_type_identifier="I",
            tax=self.tax
        )
        self.price_without_customer_group = StandardPriceFactory.create(
            product_type=self.product_without_customer_group,
            customer_group=None,
            price=100,
            unit=self.unit,
            currency=self.test_currency_with_rounding,
            valid_from=start_date,
            valid_until=end_date
        )
        self.price_without_dates = StandardPriceFactory.create(
            product_type=self.product_without_dates,
            customer_group=self.customer_group,
            valid_from=None,
            valid_until=None,
            unit=self.unit,
            currency=self.test_currency_with_rounding,
            price=80
        )
        self.price_without_date_to = StandardPriceFactory.create(
            product_type=self.product_without_date_until,
            customer_group=self.customer_group,
            valid_from=start_date,
            valid_until=None,
            unit=self.unit,
            currency=self.test_currency_with_rounding,
            price=130
        )
        self.price_without_date_from = StandardPriceFactory.create(
            product_type=self.product_without_date_from,
            customer_group=self.customer_group,
            valid_from=None,
            valid_until=end_date,
            unit=self.unit,
            currency=self.test_currency_with_rounding,
            price=50
        )
        self.price_without_date_from = StandardPriceFactory.create(
            product_type=self.product_without_currency_rounding,
            currency=self.test_currency_without_rounding,
            customer_group=self.customer_group,
            price=25,
            unit=self.unit,
            valid_from=start_date,
            valid_until=end_date
        )
        self.price_without_date_from = StandardPriceFactory.create(
            product_type=self.product_with_currency_rounding,
            currency=self.test_currency_with_rounding,
            customer_group=self.customer_group,
            price=33,
            unit=self.unit,
            valid_from=start_date,
            valid_until=end_date
        )
        self.price_with_alternative_customer_group = StandardPriceFactory.create(
            product_type=self.product_with_alternative_customer_group,
            customer_group=self.alternative_customer_group,
            valid_from=start_date,
            valid_until=end_date,
            unit=self.unit,
            currency=self.test_currency_with_rounding,
            price=80
        )
        self.customer_group_transform = StandardCustomerGroupTransformFactory.create(
            from_customer_group=self.alternative_customer_group,
            to_customer_group=self.customer_group,
            product_type=self.product_with_alternative_customer_group,
            factor=0.50
        )
        self.price_with_alternative_currency = StandardPriceFactory.create(
            product_type=self.product_with_alternative_currency,
            customer_group=self.customer_group,
            valid_from=start_date,
            valid_until=end_date,
            unit=self.unit,
            currency=self.alternative_currency,
            price=80
        )
        self.currency_transform = StandardCurrencyTransformFactory.create(
            from_currency=self.alternative_currency,
            to_currency=self.test_currency_with_rounding,
            product_type=self.product_with_alternative_currency,
            factor=0.50
        )
        self.price_with_alternative_unit = StandardPriceFactory.create(
            product_type=self.product_with_alternative_unit,
            customer_group=self.customer_group,
            valid_from=start_date,
            valid_until=end_date,
            unit=self.alternative_unit,
            currency=self.test_currency_with_rounding,
            price=80
        )
        self.currency_transform = StandardUnitTransformFactory.create(
            from_unit=self.alternative_unit,
            to_unit=self.unit,
            product_type=self.product_with_alternative_unit,
            factor=0.50
        )

    @pytest.mark.back_end_tests
    def test_calculate_document_price_without_customer_group(self):
        quote_1 = StandardQuoteFactory.create(customer=self.customer)
        StandardSalesDocumentPositionFactory.create(
            quantity=1,
            discount=0,
            product_type=self.product_without_customer_group,
            overwrite_product_price=False,
            unit=self.unit,
            sales_document=quote_1
        )
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        date_now = datetime_now.date()
        Calculations.calculate_document_price(
            document=quote_1,
            pricing_date=date_now)
        self.assertEqual(
            quote_1.last_calculated_price.__str__(), "100.00")
        self.assertEqual(
            quote_1.last_calculated_tax.__str__(), "10.00")

    @pytest.mark.back_end_tests
    def test_calculate_document_price_without_date_from(self):
        quote_2 = StandardQuoteFactory.create(customer=self.customer)
        StandardSalesDocumentPositionFactory.create(
            quantity=1,
            discount=0,
            unit=self.unit,
            product_type=self.product_without_date_from,
            overwrite_product_price=False,
            sales_document=quote_2
        )
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        date_now = datetime_now.date()
        Calculations.calculate_document_price(
            document=quote_2,
            pricing_date=date_now)
        self.assertEqual(
            quote_2.last_calculated_price.__str__(), "50.00")
        self.assertEqual(
            quote_2.last_calculated_tax.__str__(), "5.00")

    @pytest.mark.back_end_tests
    def test_calculate_document_price_without_date_until(self):
        quote_3 = StandardQuoteFactory.create(customer=self.customer)
        StandardSalesDocumentPositionFactory.create(
            quantity=1,
            discount=0,
            unit=self.unit,
            product_type=self.product_without_date_until,
            overwrite_product_price=False,
            sales_document=quote_3
        )
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        date_now = datetime_now.date()
        Calculations.calculate_document_price(
            document=quote_3,
            pricing_date=date_now)
        self.assertEqual(
            quote_3.last_calculated_price.__str__(), "130.00")
        self.assertEqual(
            quote_3.last_calculated_tax.__str__(), "13.00")

    @pytest.mark.back_end_tests
    def test_calculate_document_price_without_dates(self):
        quote_4 = StandardQuoteFactory.create(customer=self.customer)
        StandardSalesDocumentPositionFactory.create(
            quantity=1,
            discount=0,
            unit=self.unit,
            product_type=self.product_without_dates,
            overwrite_product_price=False,
            sales_document=quote_4
        )
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        date_now = datetime_now.date()
        Calculations.calculate_document_price(
            document=quote_4,
            pricing_date=date_now)
        self.assertEqual(
            quote_4.last_calculated_price.__str__(), "80.00")
        self.assertEqual(
            quote_4.last_calculated_tax.__str__(), "8.00")

    @pytest.mark.back_end_tests
    def test_calculate_document_price_with_currency_rounding(self):
        quote_5 = StandardQuoteFactory.create(
            customer=self.customer,
            currency=self.test_currency_with_rounding
        )
        StandardSalesDocumentPositionFactory.create(
            quantity=1,
            discount=7.5,
            unit=self.unit,
            product_type=self.product_with_currency_rounding,
            overwrite_product_price=False,
            sales_document=quote_5
        )
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        date_now = datetime_now.date()
        Calculations.calculate_document_price(
            document=quote_5,
            pricing_date=date_now)
        self.assertEqual(
            quote_5.last_calculated_price.__str__(), "30")
        self.assertEqual(
            quote_5.last_calculated_tax.__str__(), "3")

    @pytest.mark.back_end_tests
    def test_calculate_document_price_without_currency_rounding(self):
        quote_6 = StandardQuoteFactory.create(
            customer=self.customer,
            currency=self.test_currency_without_rounding
        )
        StandardSalesDocumentPositionFactory.create(
            quantity=1,
            discount=7.5,
            unit=self.unit,
            product_type=self.product_without_currency_rounding,
            overwrite_product_price=False,
            sales_document=quote_6
        )
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        date_now = datetime_now.date()
        Calculations.calculate_document_price(
            document=quote_6,
            pricing_date=date_now)
        self.assertEqual(
            quote_6.last_calculated_price.__str__(), "23.10")
        self.assertEqual(
            quote_6.last_calculated_tax.__str__(), "2.30")

    @pytest.mark.back_end_tests
    def test_calculate_document_price_with_document_discount(self):
        quote_7 = StandardQuoteFactory.create(
            customer=self.customer,
            currency=self.test_currency_without_rounding,
            discount=10
        )
        StandardSalesDocumentPositionFactory.create(
            quantity=1,
            discount=7.5,
            unit=self.unit,
            product_type=self.product_without_currency_rounding,
            overwrite_product_price=False,
            sales_document=quote_7
        )
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        date_now = datetime_now.date()
        Calculations.calculate_document_price(
            document=quote_7,
            pricing_date=date_now)
        self.assertEqual(
            quote_7.last_calculated_price.__str__(), "20.75")
        self.assertEqual(
            quote_7.last_calculated_tax.__str__(), "2.05")

    @pytest.mark.back_end_tests
    def test_calculate_document_with_customer_group_transform(self):
        quote_8 = StandardQuoteFactory.create(
            customer=self.customer)
        StandardSalesDocumentPositionFactory.create(
            quantity=1,
            discount=0,
            unit=self.unit,
            product_type=self.product_with_alternative_customer_group,
            overwrite_product_price=False,
            sales_document=quote_8
        )
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        date_now = datetime_now.date()
        Calculations.calculate_document_price(
            document=quote_8,
            pricing_date=date_now)
        self.assertEqual(
            quote_8.last_calculated_price.__str__(), "40.00")
        self.assertEqual(
            quote_8.last_calculated_tax.__str__(), "4.00")

    @pytest.mark.back_end_tests
    def test_calculate_document_with_currency_transform(self):
        quote_9 = StandardQuoteFactory.create(
            currency=self.test_currency_with_rounding,
            customer=self.customer)
        StandardSalesDocumentPositionFactory.create(
            quantity=1,
            discount=0,
            unit=self.unit,
            product_type=self.product_with_alternative_currency,
            overwrite_product_price=False,
            sales_document=quote_9
        )
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        date_now = datetime_now.date()
        Calculations.calculate_document_price(
            document=quote_9,
            pricing_date=date_now)
        self.assertEqual(
            quote_9.last_calculated_price.__str__(), "40")
        self.assertEqual(
            quote_9.last_calculated_tax.__str__(), "4")

    @pytest.mark.back_end_tests
    def test_calculate_document_with_unit_transform(self):
        quote_10 = StandardQuoteFactory.create(customer=self.customer)
        StandardSalesDocumentPositionFactory.create(
            quantity=1,
            discount=0,
            unit=self.unit,
            product_type=self.product_with_alternative_unit,
            overwrite_product_price=False,
            sales_document=quote_10
        )
        datetime_now = make_date_utc(datetime.datetime(2024, 1, 1, 0, 00))
        date_now = datetime_now.date()
        Calculations.calculate_document_price(
            document=quote_10,
            pricing_date=date_now)
        self.assertEqual(
            quote_10.last_calculated_price.__str__(), "40.00")
        self.assertEqual(
            quote_10.last_calculated_tax.__str__(), "4.00")



# coding: utf-8

# DJANGO IMPORTS
from django.urls import path

from koalixcrm.crm.views.time_tracking import work_report
from koalixcrm.crm.views.user_extension_missing import user_extension_missing
from koalixcrm.crm.views.reporting_period_missing import reporting_period_missing
from koalixcrm.crm.views.user_is_not_human_resource import user_is_not_human_resource
from koalixcrm.crm.views.set_timezone import set_timezone

urlpatterns = [
    path('time_tracking/', work_report, name="monthly_report"),
    path('user_extension_missing/', user_extension_missing, name="user_extension_missing"),
    path('reporting_period_missing/', reporting_period_missing, name="reporting_period_missing"),
    path('user_is_not_human_resource/', user_is_not_human_resource, name="user_is_not_human_resource"),
    path('set_timezone/', set_timezone, name="set_timezone"),
]


# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-22 21:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0023_auto_20180119_2102'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='position',
            options={'ordering': ['position_number'], 'verbose_name': 'Position', 'verbose_name_plural': 'Positions'},
        ),
        migrations.AddField(
            model_name='salesdocument',
            name='custom_date_field',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Custom Date/Time'),
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import models
from django.forms import ValidationError
from django.contrib import admin
from django.utils.html import format_html
from django.utils.translation import ugettext as _
from koalixcrm.crm.documents.pdf_export import PDFExport
from koalixcrm.global_support_functions import *
from koalixcrm.crm.exceptions import ReportingPeriodDoneDeleteNotPossible
from django.contrib import messages


class Work(models.Model):
    human_resource = models.ForeignKey("HumanResource", on_delete=models.CASCADE)
    date = models.DateField(verbose_name=_("Date"),
                            blank=False,
                            null=False)
    start_time = models.DateTimeField(verbose_name=_("Start Time"),
                                      blank=True,
                                      null=True)
    stop_time = models.DateTimeField(verbose_name=_("Stop Time"),
                                     blank=True,
                                     null=True)
    worked_hours = models.DecimalField(verbose_name=_("Worked Hours"),
                                       max_digits=5,
                                       decimal_places=2,
                                       blank=True,
                                       null=True)
    short_description = models.CharField(verbose_name=_("Short Description"),
                                         max_length=300,
                                         blank=False,
                                         null=False)
    description = models.TextField(verbose_name=_("Text"),
                                   blank=True,
                                   null=True)
    task = models.ForeignKey("Task",
                             on_delete=models.CASCADE,
                             verbose_name=_('Task'),
                             blank=False,
                             null=False)
    reporting_period = models.ForeignKey("ReportingPeriod",
                                         on_delete=models.CASCADE,
                                         verbose_name=_('Reporting Period'),
                                         blank=False,
                                         null=False)

    def link_to_work(self):
        if self.id:
            return format_html("<a href='/admin/crm/work/%s' >%s</a>" % (str(self.id), str(self.id)))
        else:
            return "Not present"
    link_to_work.short_description = _("Work")

    def get_short_description(self):
        if self.short_description:
            return self.short_description
        elif self.description:
            return limit_string_length(self.description, 100)
        else:
            return _("Please add description")
    get_short_description.short_description = _("Short description");

    def serialize_to_xml(self):
        objects = [self, ]
        main_xml = PDFExport.write_xml(objects)
        return main_xml

    def effort_hours(self):
        if self.effort_seconds() != 0:
            return self.effort_seconds()/3600
        else:
            return 0

    def effort_seconds(self):
        if not self.start_stop_pattern_complete() and not bool(self.worked_hours):
            return 0
        elif (not bool(self.stop_time)) or (not bool(self.start_time)):
            return float(self.worked_hours)*3600
        else:
            return (self.stop_time - self.start_time).total_seconds()

    def effort_as_string(self):
        return str(self.effort_hours()) + " h"

    def start_stop_pattern_complete(self):
        return bool(self.start_time) & bool(self.stop_time)

    def start_stop_pattern_stop_missing(self):
        return bool(self.start_time) & (not bool(self.stop_time))

    def start_stop_pattern_start_missing(self):
        return bool(self.stop_time) & (not bool(self.start_time))

    def __str__(self):
        return _("Work") + ": " + str(self.id) + " " + _("from Person") + ": " + str(self.human_resource.id)

    def check_working_hours(self):
        """This method checks that the working hour is correctly provided either using the start_stop pattern
        or by providing the worked_hours in total.

        Args:

        Returns:
          True when no ValidationError was raised

        Raises:
          may raise ValidationError exception"""
        if self.start_stop_pattern_complete() & bool(self.worked_hours):
            raise ValidationError('Please either set the start, stop time or worked hours (not both)', code='invalid')
        elif self.start_stop_pattern_start_missing() or self.start_stop_pattern_stop_missing():
            raise ValidationError('Set start and stop time', code='invalid')
        return True

    def clean(self):
        cleaned_data = super(Work, self).clean()
        self.check_working_hours()
        return cleaned_data

    def confirmed(self):
        return self.reporting_period.status.is_done
    confirmed.short_description = _("Confirmed")
    confirmed.tags = True

    def delete(self, using=None, keep_parents=False):
        if self.confirmed():
            raise ReportingPeriodDoneDeleteNotPossible("It was not possible to delete the work")
        else:
            super(Work, self).delete(using, keep_parents)

    def save(self, *args, **kwargs):
        if self.confirmed():
            raise ReportingPeriodDoneDeleteNotPossible("It was not possible to update the work")
        else:
            super(Work, self).save(*args, **kwargs)

    class Meta:
        app_label = "crm"
        verbose_name = _('Work')
        verbose_name_plural = _('Work')


class WorkAdminView(admin.ModelAdmin):
    list_display = ('link_to_work',
                    'human_resource',
                    'task',
                    'get_short_description',
                    'date',
                    'reporting_period',
                    'effort_as_string',
                    'confirmed')

    list_filter = ('task', 'date')
    ordering = ('-id',)

    fieldsets = (
        (_('Work'), {
            'fields': ('human_resource',
                       'date',
                       'start_time',
                       'stop_time',
                       'worked_hours',
                       'short_description',
                       'description',
                       'task',
                       'reporting_period')
        }),
    )
    save_as = True

    actions = ['delete_work', ]

    def delete_work(self, request, queryset):
        for obj in queryset:
            if obj.reporting_period.status.is_done:
                self.message_user(request, _("Delete is not allowed because the work"
                                             " is used in a reporting period which is marked "
                                             "'as done'"),
                                  level=messages.ERROR)
            else:
                obj.delete()

    delete_work.short_description = _("Delete Selected Work")


class WorkInlineAdminView(admin.TabularInline):
    model = Work
    readonly_fields = ('link_to_work',
                       'get_short_description',
                       'human_resource',
                       'date',
                       'effort_as_string',
                       'confirmed',
                       'task',
                       'reporting_period',)
    fieldsets = (
        (_('Work'), {
            'fields': ('link_to_work',
                       'get_short_description',
                       'human_resource',
                       'date',
                       'effort_as_string',
                       'confirmed',
                       'task',
                       'reporting_period',)
        }),
    )
    extra = 0

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _

from koalixcrm.plugin import *
from koalixcrm.crm.contact.phone_address import PhoneAddress
from koalixcrm.crm.contact.email_address import EmailAddress
from koalixcrm.crm.contact.postal_address import PostalAddress
from koalixcrm.crm.documents.invoice import Invoice
from koalixcrm.crm.documents.quote import Quote
from koalixcrm.crm.documents.purchase_order import PurchaseOrder
from koalixcrm.global_support_functions import xstr
from koalixcrm.crm.const.purpose import *
from koalixcrm.crm.documents.invoice import InlineInvoice
from koalixcrm.crm.documents.quote import InlineQuote
from koalixcrm.crm.reporting.generic_project_link import InlineGenericProjectLink
from koalixcrm.crm.exceptions import *
from koalixcrm.djangoUserExtension.models import UserExtension
import koalixcrm.crm.documents.calculations
import koalixcrm.crm.documents.pdf_export
from rest_framework import serializers


class PostalAddressForContract(PostalAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    contract = models.ForeignKey('Contract', on_delete=models.CASCADE)

    class Meta:
        app_label = "crm"
        verbose_name = _('Postal Address For Contracts')
        verbose_name_plural = _('Postal Address For Contracts')

    def __str__(self):
        return xstr(self.prename) + ' ' + xstr(self.name) + ' ' + xstr(self.addressline1)


class PhoneAddressForContract(PhoneAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    contract = models.ForeignKey('Contract', on_delete=models.CASCADE)

    class Meta:
        app_label = "crm"
        verbose_name = _('Phone Address For Contracts')
        verbose_name_plural = _('Phone Address For Contracts')

    def __str__(self):
        return str(self.phone)


class EmailAddressForContract(EmailAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    contract = models.ForeignKey('Contract', on_delete=models.CASCADE)

    class Meta:
        app_label = "crm"
        verbose_name = _('Email Address For Contracts')
        verbose_name_plural = _('Email Address For Contracts')

    def __str__(self):
        return str(self.email)


class ContractPostalAddress(admin.StackedInline):
    model = PostalAddressForContract
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('prefix',
                       'pre_name',
                       'name',
                       'address_line_1',
                       'address_line_2',
                       'address_line_3',
                       'address_line_4',
                       'zip_code',
                       'town',
                       'state',
                       'country',
                       'purpose'),
        }),
    )
    allow_add = True


class ContractPhoneAddress(admin.TabularInline):
    model = PhoneAddressForContract
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('phone', 'purpose',)
        }),
    )
    allow_add = True


class ContractEmailAddress(admin.TabularInline):
    model = EmailAddressForContract
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('email',
                       'purpose',)
        }),
    )
    allow_add = True


class Contract(models.Model):
    staff = models.ForeignKey('auth.User',
                              on_delete=models.CASCADE,
                              limit_choices_to={'is_staff': True},
                              verbose_name=_("Staff"),
                              related_name="db_relcontractstaff",
                              blank=True,
                              null=True)
    description = models.TextField(verbose_name=_("Description"))
    default_customer = models.ForeignKey("Customer",
                                         on_delete=models.CASCADE,
                                         verbose_name=_("Default Customer"),
                                         null=True,
                                         blank=True)
    default_supplier = models.ForeignKey("Supplier",
                                         on_delete=models.CASCADE,
                                         verbose_name=_("Default Supplier"),
                                         null=True,
                                         blank=True)
    default_currency = models.ForeignKey("Currency",
                                         on_delete=models.CASCADE,
                                         verbose_name=_("Default Currency"),
                                         blank=False,
                                         null=False)
    default_template_set = models.ForeignKey("djangoUserExtension.TemplateSet",
                                             on_delete=models.CASCADE,
                                             verbose_name=_("Default Template Set"),
                                             null=True,
                                             blank=True)
    date_of_creation = models.DateTimeField(verbose_name=_("Created at"),
                                            auto_now_add=True)
    last_modification = models.DateTimeField(verbose_name=_("Last modified"),
                                             auto_now=True)
    last_modified_by = models.ForeignKey('auth.User',
                                         on_delete=models.CASCADE,
                                         limit_choices_to={'is_staff': True},
                                         verbose_name=_("Last modified by"),
                                         related_name="db_contractlstmodified")

    class Meta:
        app_label = "crm"
        verbose_name = _('Contract')
        verbose_name_plural = _('Contracts')

    def get_template_set(self, calling_model):
        if self.default_template_set:
            required_template_set = str(type(calling_model).__name__)
            return self.default_template_set.get_template_set(required_template_set)
        else:
            raise TemplateSetMissingInContract("The Contract has no Default Template Set selected")

    def create_from_reference(self, calling_model, staff):
        staff_user_extension = UserExtension.get_user_extension(staff.id)
        self.default_customer = calling_model
        self.default_currency = staff_user_extension.defaultCurrency
        self.default_template_set = staff_user_extension.defaultTemplateSet
        self.last_modified_by = staff
        self.staff = staff
        self.save()
        return self

    def create_invoice(self):
        invoice = Invoice()
        invoice.create_from_reference(self)
        return invoice

    def create_quote(self):
        quote = Quote()
        quote.create_from_reference(self)
        return quote

    def create_purchase_order(self):
        purchase_order = PurchaseOrder()
        purchase_order.create_from_reference(self)
        return purchase_order

    def __str__(self):
        return _("Contract") + " " + str(self.id)


class OptionContract(admin.ModelAdmin):
    list_display = ('id',
                    'description',
                    'default_customer',
                    'default_supplier',
                    'staff',
                    'default_currency',
                    'date_of_creation',
                    'last_modification',
                    'last_modified_by')
    list_display_links = ('id',)
    list_filter = ('default_customer',
                   'default_supplier',
                   'staff',
                   'default_currency')
    ordering = ('id', )
    search_fields = ('id',
                     'contract')
    fieldsets = (
        (_('Basics'), {
            'fields': ('description',
                       'default_customer',
                       'staff',
                       'default_supplier',
                       'default_currency',
                       'default_template_set')
        }),
    )
    inlines = [ContractPostalAddress,
               ContractPhoneAddress,
               ContractEmailAddress,
               InlineQuote,
               InlineInvoice,
               InlineGenericProjectLink]
    pluginProcessor = PluginProcessor()
    inlines.extend(pluginProcessor.getPluginAdditions("contractInlines"))

    def create_quote(self, request, queryset):
        from koalixcrm.crm.views.newdocument import CreateNewDocumentView
        for obj in queryset:
            response = CreateNewDocumentView.create_new_document(self,
                                                                 request,
                                                                 obj,
                                                                 koalixcrm.crm.documents.quote.Quote,
                                                                 ("/admin/crm/"+obj.__class__.__name__.lower()+"/"))
            return response

    create_quote.short_description = _("Create Quote")

    def create_invoice(self, request, queryset):
        from koalixcrm.crm.views.newdocument import CreateNewDocumentView
        for obj in queryset:
            response = CreateNewDocumentView.create_new_document(self,
                                                                 request,
                                                                 obj,
                                                                 koalixcrm.crm.documents.invoice.Invoice,
                                                                 ("/admin/crm/"+obj.__class__.__name__.lower()+"/"))
            return response

    create_invoice.short_description = _("Create Invoice")

    def create_purchase_confirmation(self, request, queryset):
        from koalixcrm.crm.views.newdocument import CreateNewDocumentView
        for obj in queryset:
            response = CreateNewDocumentView.create_new_document(self,
                                                                 request,
                                                                 obj,
                                                                 koalixcrm.crm.documents.purchase_confirmation.PurchaseConfirmation,
                                                                 ("/admin/crm/"+obj.__class__.__name__.lower()+"/"))
            return response

    create_purchase_confirmation.short_description = _("Create Purchase Confirmation")

    def create_delivery_note(self, request, queryset):
        from koalixcrm.crm.views.newdocument import CreateNewDocumentView
        for obj in queryset:
            response = CreateNewDocumentView.create_new_document(self,
                                                                 request,
                                                                 obj,
                                                                 koalixcrm.crm.documents.delivery_note.DeliveryNote,
                                                                 ("/admin/crm/"+obj.__class__.__name__.lower()+"/"))
            return response

    create_delivery_note.short_description = _("Create Delivery note")

    def create_payment_reminder(self, request, queryset):
        from koalixcrm.crm.views.newdocument import CreateNewDocumentView
        for obj in queryset:
            response = CreateNewDocumentView.create_new_document(self,
                                                                 request,
                                                                 obj,
                                                                 koalixcrm.crm.documents.payment_reminder.PaymentReminder,
                                                                 ("/admin/crm/"+obj.__class__.__name__.lower()+"/"))
            return response

    create_payment_reminder.short_description = _("Create Payment Reminder")

    def create_purchase_order(self, request, queryset):
        from koalixcrm.crm.views.newdocument import CreateNewDocumentView
        for obj in queryset:
            response = CreateNewDocumentView.create_new_document(self,
                                                                 request,
                                                                 obj,
                                                                 koalixcrm.crm.documents.purchase_order.PurchaseOrder,
                                                                 ("/admin/crm/"+obj.__class__.__name__.lower()+"/"))
            return response

    create_purchase_order.short_description = _("Create Purchase Order")

    def save_model(self, request, obj, form, change):
        if change:
            obj.last_modified_by = request.user
        else:
            obj.last_modified_by = request.user
            obj.staff = request.user
        obj.save()

    actions = ['create_quote',
               'create_invoice',
               'create_purchase_order']
    pluginProcessor = PluginProcessor()
    actions.extend(pluginProcessor.getPluginAdditions("contractActions"))


class ContractJSONSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = Contract
        fields = ('id',
                  'description')



import datetime
import pytest
from django.test import TestCase
from koalixcrm.crm.factories.factory_user import AdminUserFactory
from koalixcrm.crm.factories.factory_customer_billing_cycle import StandardCustomerBillingCycleFactory
from koalixcrm.crm.factories.factory_customer import StandardCustomerFactory
from koalixcrm.crm.factories.factory_customer_group import StandardCustomerGroupFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.crm.factories.factory_reporting_period import StandardReportingPeriodFactory
from koalixcrm.djangoUserExtension.factories.factory_user_extension import StandardUserExtensionFactory
from koalixcrm.crm.factories.factory_task import StandardTaskFactory
from koalixcrm.crm.factories.factory_estimation import StandardEstimationToTaskFactory
from koalixcrm.crm.factories.factory_human_resource import StandardHumanResourceFactory
from koalixcrm.crm.factories.factory_resource_price import StandardResourcePriceFactory
from koalixcrm.crm.factories.factory_unit import StandardUnitFactory


class TaskPlannedEffort(TestCase):
    def setUp(self):
        self.test_billing_cycle = StandardCustomerBillingCycleFactory.create()
        self.test_user = AdminUserFactory.create()
        self.test_customer_group = StandardCustomerGroupFactory.create()
        self.test_unit = StandardUnitFactory.create()
        self.test_customer = StandardCustomerFactory.create(is_member_of=(self.test_customer_group,))
        self.test_currency = StandardCurrencyFactory.create()
        self.test_user_extension = StandardUserExtensionFactory.create(user=self.test_user)
        self.test_reporting_period = StandardReportingPeriodFactory.create()
        self.test_human_resource = StandardHumanResourceFactory.create()
        self.resource_price = StandardResourcePriceFactory.create(
            resource=self.test_human_resource,
            unit=self.test_unit,
            currency=self.test_currency,
            customer_group=self.test_customer_group,
            price="120",
        )
        self.test_1st_task = StandardTaskFactory.create(title="1st Test Task",
                                                        project=self.test_reporting_period.project)
        self.test_2nd_task = StandardTaskFactory.create(title="2nd Test Task",
                                                        project=self.test_reporting_period.project)

    @pytest.mark.back_end_tests
    def test_task_planned_costs(self):
        StandardEstimationToTaskFactory.create(resource=self.test_human_resource,
                                               amount="2.00",
                                               task=self.test_1st_task,
                                               reporting_period=self.test_reporting_period)
        StandardEstimationToTaskFactory.create(resource=self.test_human_resource,
                                               amount="4.75",
                                               task=self.test_2nd_task,
                                               reporting_period=self.test_reporting_period)
        self.assertEqual(
            (self.test_1st_task.planned_effort()).__str__(), "2.00")
        self.assertEqual(
            (self.test_1st_task.planned_costs()).__str__(), "240.0000")
        self.assertEqual(
            (self.test_2nd_task.planned_effort()).__str__(), "4.75")
        self.assertEqual(
            (self.test_2nd_task.planned_costs()).__str__(), "570.0000")



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import SalesDocumentPosition
from koalixcrm.crm.factories.factory_product_type import StandardProductTypeFactory
from koalixcrm.crm.factories.factory_unit import StandardUnitFactory


class StandardSalesDocumentPositionFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = SalesDocumentPosition

    position_number = 5
    quantity = 5
    description = "This is a test sales document position"
    discount = 10
    product_type = factory.SubFactory(StandardProductTypeFactory)
    unit = factory.SubFactory(StandardUnitFactory)
    overwrite_product_price = True
    position_price_per_unit = 155




# -*- coding: utf-8 -*-
from django.contrib import admin
from django.db import models
from django.utils.translation import ugettext as _


class Booking(models.Model):
    from_account = models.ForeignKey('Account', on_delete=models.CASCADE, verbose_name=_("From Account"), related_name="db_booking_fromaccount")
    to_account = models.ForeignKey('Account', on_delete=models.CASCADE, verbose_name=_("To Account"), related_name="db_booking_toaccount")
    amount = models.DecimalField(max_digits=20, decimal_places=2, verbose_name=_("Amount"))
    description = models.CharField(verbose_name=_("Description"), max_length=120, null=True, blank=True)
    booking_reference = models.ForeignKey('crm.Invoice', on_delete=models.CASCADE, verbose_name=_("Booking Reference"), null=True, blank=True)
    booking_date = models.DateTimeField(verbose_name=_("Booking at"))
    accounting_period = models.ForeignKey('AccountingPeriod', on_delete=models.CASCADE, verbose_name=_("AccountingPeriod"))
    staff = models.ForeignKey('auth.User', on_delete=models.CASCADE, limit_choices_to={'is_staff': True}, blank=True,
                              verbose_name=_("Reference Staff"), related_name="db_booking_refstaff")
    date_of_creation = models.DateTimeField(verbose_name=_("Created at"), auto_now=True)
    last_modification = models.DateTimeField(verbose_name=_("Last modified"), auto_now_add=True)
    last_modified_by = models.ForeignKey('auth.User', on_delete=models.CASCADE, limit_choices_to={'is_staff': True}, blank=True,
                                         verbose_name=_("Last modified by"), related_name="db_booking_lstmodified")

    def booking_date_only(self):
        return self.booking_date.date()

    booking_date_only.short_description = _("Date");

    def __str__(self):
        return self.from_account.__str__() + " " + self.to_account.__str__() + " " + self.amount.__str__()

    class Meta:
        app_label = "accounting"
        verbose_name = _('Booking')
        verbose_name_plural = _('Bookings')


class OptionBooking(admin.ModelAdmin):
    list_display = ('from_account',
                    'to_account',
                    'amount',
                    'booking_date_only',
                    'staff')
    fieldsets = ((_('Basic'), {'fields': ('from_account',
                                          'to_account',
                                          'amount',
                                          'booking_date',
                                          'staff',
                                          'description',
                                          'booking_reference',
                                          'accounting_period')}),)
    save_as = True

    def save_model(self, request, obj, form, change):
        if change:
            obj.last_modified_by = request.user
        else:
            obj.last_modified_by = request.user
            obj.staff = request.user
        obj.save()


class InlineBookings(admin.TabularInline):
    model = Booking
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('from_account',
                       'to_account',
                       'description',
                       'amount',
                       'booking_date',
                       'staff',
                       'booking_reference',)
        }),
    )
    allow_add = False



# -*- coding: utf-8 -*-

import factory
import datetime
from koalixcrm.crm.models import Agreement
from koalixcrm.crm.factories.factory_resource import StandardResourceFactory
from koalixcrm.crm.factories.factory_human_resource import StandardHumanResourceFactory
from koalixcrm.crm.factories.factory_task import StandardTaskFactory
from koalixcrm.crm.factories.factory_unit import StandardUnitFactory
from koalixcrm.crm.factories.factory_resource_price import StandardResourcePriceFactory
from koalixcrm.crm.factories.factory_agreement_type import StandardAgreementTypeFactory
from koalixcrm.crm.factories.factory_agreement_status import AgreedAgreementStatusFactory
from koalixcrm.global_support_functions import make_date_utc


class StandardAgreementToTaskFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Agreement

    date_from = make_date_utc(datetime.datetime(2018, 6, 15, 00))
    date_until = make_date_utc(datetime.datetime(2024, 6, 15, 00))
    amount = "112.50"
    task = factory.SubFactory(StandardTaskFactory)
    resource = factory.SubFactory(StandardResourceFactory)
    unit = factory.SubFactory(StandardUnitFactory)
    status = factory.SubFactory(AgreedAgreementStatusFactory)
    costs = factory.SubFactory(StandardResourcePriceFactory)
    type = factory.SubFactory(StandardAgreementTypeFactory)


class StandardHumanResourceAgreementToTaskFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Agreement

    date_from = make_date_utc(datetime.datetime(2018, 6, 15, 00))
    date_until = make_date_utc(datetime.datetime(2024, 6, 15, 00))
    amount = "112.50"
    task = factory.SubFactory(StandardTaskFactory)
    resource = factory.SubFactory(StandardHumanResourceFactory)
    unit = factory.SubFactory(StandardUnitFactory)
    status = factory.SubFactory(AgreedAgreementStatusFactory)
    costs = factory.SubFactory(StandardResourcePriceFactory)
    type = factory.SubFactory(StandardAgreementTypeFactory)



# -*- coding: utf-8 -*-

from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _
from koalixcrm.crm.reporting.resource_price import ResourcePrice


class Agreement(models.Model):
    """The Agreement describes the contract between the steer-co or the customer with the project manager"""
    task = models.ForeignKey("Task",
                             on_delete=models.CASCADE,
                             verbose_name=_('Task'),
                             blank=False,
                             null=False)
    resource = models.ForeignKey("Resource", on_delete=models.CASCADE)
    unit = models.ForeignKey("Unit", on_delete=models.CASCADE)
    costs = models.ForeignKey(ResourcePrice, on_delete=models.CASCADE)
    type = models.ForeignKey("AgreementType", on_delete=models.CASCADE)
    status = models.ForeignKey("AgreementStatus", on_delete=models.CASCADE)
    date_from = models.DateField(verbose_name=_("Agreement From"),
                                 blank=False,
                                 null=False)
    date_until = models.DateField(verbose_name=_("Agreement To"),
                                  blank=False,
                                  null=False)
    amount = models.DecimalField(verbose_name=_("Amount"),
                                 max_digits=5,
                                 decimal_places=2,
                                 blank=True,
                                 null=True)

    def calculated_costs(self):
        return 0

    def match_with_work(self, work):
        """This method checks whether the provided work can be covered by the agreement.
        the method checks whether the reported work corresponds with the resource and whether the
        reported work was within the time-frame of the agreement.
        The method returns False when the Agreement is not yet in status agreed

        Args:
          work (koalixcrm.crm.reporting.work.Work)

        Returns:
          True when no ValidationError was raised

        Raises:
          should not raise exceptions"""
        matches = False
        if self.status.is_agreed:
            if (work.date >= self.date_from) and (work.date <= self.date_until):
                if self.resource.id == work.human_resource.id:
                    matches = True
        return matches

    def __str__(self):
        return _("Agreement of Resource Consumption") + ": " + str(self.id)

    class Meta:
        app_label = "crm"
        verbose_name = _('Agreement')
        verbose_name_plural = _('Agreements')


class AgreementInlineAdminView(admin.TabularInline):
    model = Agreement
    fieldsets = (
        (_('Work'), {
            'fields': ('task',
                       'resource',
                       'amount',
                       'unit',
                       'costs',
                       'date_from',
                       'date_until',
                       'type',
                       'status')
        }),
    )
    extra = 1



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-10-12 20:56
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0049_auto_20181014_2258'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='customergrouptransform',
            name='product',
        ),
        migrations.RemoveField(
            model_name='position',
            name='product',
        ),
        migrations.RemoveField(
            model_name='unittransform',
            name='product',
        ),
        migrations.RemoveField(
            model_name='Price',
            name='product',
        ),
        migrations.RenameModel(old_name='Product',
                               new_name='ProductType'),

        migrations.CreateModel(
            name='ProductPrice',
            fields=[
                ('price_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='crm.Price')),
                ('product_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.ProductType', verbose_name='Product'),)
            ],
            options={
                'verbose_name': 'Product Price',
                'verbose_name_plural': 'Product Prices',
            },
            bases=('crm.price',),
        ),
        migrations.AddField(
            model_name='customergrouptransform',
            name='product_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='crm.ProductType', verbose_name='Product Type'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='position',
            name='product_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='crm.ProductType', verbose_name='Product'),
        ),
        migrations.AddField(
            model_name='unittransform',
            name='product_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='crm.ProductType', verbose_name='Product Type'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-

import os
from decimal import *
from datetime import *
from django.conf import settings
from django.db import models
from django.contrib import admin
from django.utils.translation import ugettext as _
from django.utils.html import format_html
from koalixcrm.crm.reporting.generic_project_link import GenericLinkInlineAdminView
from koalixcrm.crm.reporting.reporting_period import ReportingPeriodInlineAdminView, ReportingPeriod
from koalixcrm.crm.reporting.task import TaskInlineAdminView
from koalixcrm.crm.documents.pdf_export import PDFExport
from koalixcrm.crm.exceptions import TemplateSetMissingInContract
from koalixcrm.crm.models import Task
import matplotlib.dates as mdates
from matplotlib import pyplot
import pandas


class Project(models.Model):
    project_manager = models.ForeignKey('auth.User', on_delete=models.CASCADE, limit_choices_to={'is_staff': True},
                                        verbose_name=_("Staff"),
                                        related_name="db_rel_project_staff",
                                        blank=True,
                                        null=True)
    project_name = models.CharField(verbose_name=_("Project name"),
                                    max_length=100,
                                    null=True,
                                    blank=True)
    description = models.TextField(verbose_name=_("Description"),
                                   null=True,
                                   blank=True)
    project_status = models.ForeignKey("ProjectStatus",
                                       on_delete=models.CASCADE,
                                       verbose_name=_('Project Status'),
                                       blank=True,
                                       null=True)
    default_template_set = models.ForeignKey("djangoUserExtension.TemplateSet",
                                             on_delete=models.CASCADE,
                                             verbose_name=_("Default Template Set"),
                                             null=True,
                                             blank=True)
    default_currency = models.ForeignKey("Currency",
                                         on_delete=models.CASCADE,
                                         verbose_name=_("Default Currency"),
                                         null=False,
                                         blank=False)
    date_of_creation = models.DateTimeField(verbose_name=_("Created at"),
                                            auto_now_add=True)
    last_modification = models.DateTimeField(verbose_name=_("Last modified"),
                                             auto_now=True)
    last_modified_by = models.ForeignKey('auth.User',
                                         on_delete=models.CASCADE,
                                         limit_choices_to={'is_staff': True},
                                         verbose_name=_("Last modified by"),
                                         related_name="db_project_last_modified")

    def link_to_project(self):
        if self.id:
            return format_html("<a href='/admin/crm/project/%s' >%s</a>" % (str(self.id), str(self.project_name)))
        else:
            return "Not present"
    link_to_project.short_description = _("Project")

    def create_pdf(self, template_set, printed_by):
        self.last_print_date = datetime.now()
        self.save()
        return PDFExport.create_pdf(self, template_set, printed_by)

    def get_template_set(self):
        if self.default_template_set.monthly_project_summary_template:
            return self.default_template_set.monthly_project_summary_template
        else:
            raise TemplateSetMissingInContract((_("Template Set missing in Project" + str(self))))

    def get_fop_config_file(self, template_set):
        template_set = self.get_template_set()
        return template_set.get_fop_config_file()

    def get_xsl_file(self, template_set):
        template_set = self.get_template_set()
        return template_set.get_xsl_file()

    def get_reporting_period(self, search_date):
        from koalixcrm.crm.reporting.reporting_period import ReportingPeriod
        """Returns the reporting period that is valid. Valid is a reporting period when the provided date
          lies between begin and end of the reporting period

        Args:
          no arguments

        Returns:
          accounting_period (ReportPeriod)

        Raises:
          ReportPeriodNotFound when there is no valid reporting Period"""
        return ReportingPeriod.get_reporting_period(self, search_date)

    def serialize_to_xml(self, **kwargs):
        reporting_period = kwargs.get('reporting_period', None)
        from koalixcrm.djangoUserExtension.models import UserExtension
        objects = [self, ]
        objects += UserExtension.objects_to_serialize(self, self.project_manager)
        main_xml = PDFExport.write_xml(objects)
        for task in Task.objects.filter(project=self.id):
            task_xml = task.serialize_to_xml(reporting_period=reporting_period)
            main_xml = PDFExport.merge_xml(main_xml, task_xml)
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='crm.project']",
                                                       "Effective_Costs_Confirmed",
                                                       self.effective_costs_confirmed())
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='crm.project']",
                                                       "Effective_Costs_Not_Confirmed",
                                                       self.effective_costs_not_confirmed())
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='crm.project']",
                                                       "Effective_Effort_Overall",
                                                       self.effective_effort(reporting_period=None))
        if reporting_period:
            main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                           "object/[@model='crm.project']",
                                                           "Effective_Costs_InPeriod",
                                                           self.effective_costs(reporting_period=reporting_period))
            main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                           "object/[@model='crm.project']",
                                                           "Effective_Effort_InPeriod",
                                                           self.effective_effort(reporting_period=reporting_period))
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='crm.project']",
                                                       "Planned_Total_Costs",
                                                       self.planned_total_costs())
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='crm.project']",
                                                       "Effective_Duration",
                                                       self.effective_duration())
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='crm.project']",
                                                       "Planned_Duration",
                                                       self.planned_duration())
        main_xml = PDFExport.append_element_to_pattern(main_xml,
                                                       "object/[@model='crm.project']",
                                                       "project_cost_overview",
                                                       self.create_project_cost_overview_illustration())
        return main_xml

    def create_project_cost_overview_illustration(self, reporting_period=None):
        """The function return a link to a svg illustration containing the cost overview of the project

        Args:
        reporting_period (ReportingPeriod)

        Returns:
        planned costs (String)

        Raises:
        No exceptions planned"""
        path_to_illustration = os.path.join(settings.PDF_OUTPUT_ROOT + "/project_costs_overview.svg")
        data_frame = None
        effective_costs = 0
        accumulated_effective_cost = 0
        accumulated_effective_costs = dict()
        accumulated_planned_costs = dict()
        reporting_periods = ReportingPeriod.objects.filter(project=self.id).order_by('begin')
        for reporting_period in reporting_periods:
            effective_costs = self.effective_costs(reporting_period=reporting_period)
            accumulated_effective_cost += effective_costs
            accumulated_effective_costs[reporting_period] = accumulated_effective_cost
        accumulated_planned_costs = self.planned_costs_in_buckets(reporting_period=reporting_period,
                                                                  buckets=reporting_periods)
        for reporting_period in reporting_periods:
            if reporting_period.status.is_done:
                effective_confirmed_costs_this_bucket = int(accumulated_effective_costs[reporting_period])
                effective_not_confirmed_costs_this_bucket = int(accumulated_effective_costs[reporting_period])
            else:
                effective_confirmed_costs_this_bucket = None
                effective_not_confirmed_costs_this_bucket = int(accumulated_effective_costs[reporting_period])
            if data_frame is None:
                data_frame = pandas.DataFrame([[reporting_period.begin,
                                                None,
                                                0,
                                                0,
                                                0],
                                               [reporting_period.end,
                                                None,
                                                int(accumulated_planned_costs[reporting_period]),
                                                effective_confirmed_costs_this_bucket,
                                                effective_not_confirmed_costs_this_bucket], ],
                                              columns=('x',
                                                       'Budget',
                                                       'Estimation',
                                                       'Effective confirmed',
                                                       'Effective not confirmed'))
            else:
                data_frame_to_add = pandas.DataFrame([[reporting_period.end,
                                                       None,
                                                       int(accumulated_planned_costs[reporting_period]),
                                                       effective_confirmed_costs_this_bucket,
                                                       effective_not_confirmed_costs_this_bucket], ],
                                                     columns=('x',
                                                              'Budget',
                                                              'Estimation',
                                                              'Effective confirmed',
                                                              'Effective not confirmed'))
                data_frame = data_frame.append(data_frame_to_add, ignore_index=False)

        pyplot.style.use('seaborn-darkgrid')
        figure, axis = pyplot.subplots()
        axis.plot(data_frame['x'], data_frame.get("Budget"),
                  marker=' ',
                  color="red",
                  linewidth=1,
                  alpha=0.9,
                  label="Agreed Budget")

        axis.plot(data_frame['x'], data_frame.get("Estimation"),
                  marker='o',
                  color="orangered",
                  linewidth=2,
                  alpha=0.5,
                  label="Estimation (accumulated)")

        axis.plot(data_frame['x'], data_frame.get("Effective confirmed"),
                  marker='o',
                  color="orange",
                  linewidth=2,
                  alpha=0.5,
                  label="Effective confirmed (accumulated)")
        axis.plot(data_frame['x'], data_frame.get("Effective not confirmed"),
                  marker='o',
                  color="gold",
                  linewidth=2,
                  alpha=0.5,
                  label="Effective not confirmed (accumulated)")

        axis.legend(loc=2, ncol=1)
        figure.autofmt_xdate()
        axis.fmt_xdata = mdates.DateFormatter('%Y-%m-%d')
        axis.set_title("Project Costs Overview", loc='left', fontsize=12, fontweight=0, color='orange')
        axis.set_xlabel("Date")
        axis.set_ylabel("Costs in " + self.default_currency.__str__())
        figure.savefig(path_to_illustration)
        figure.clf()

        return path_to_illustration

    def effective_costs(self, reporting_period=None, confirmed=False):
        effective_cost = 0
        for task in Task.objects.filter(project=self.id):
            effective_cost += task.effective_costs(reporting_period=reporting_period, confirmed=confirmed)
        self.default_currency.round(effective_cost)
        return effective_cost

    def effective_costs_confirmed(self):
        return self.effective_costs(confirmed=True)
    effective_costs_confirmed.short_description = _("Effective Confirmed Costs")
    effective_costs_confirmed.tags = True

    def effective_costs_not_confirmed(self):
        return self.effective_costs(confirmed=False)
    effective_costs_not_confirmed.short_description = _("Effective Not Confirmed Costs")
    effective_costs_not_confirmed.tags = True

    def effective_effort(self, reporting_period=None):
        effective_effort = 0
        for task in Task.objects.filter(project=self.id):
            effective_effort += task.effective_effort(reporting_period=reporting_period)
        return effective_effort
    effective_effort.short_description = _("Effective Accumulated effort")
    effective_effort.tags = True

    def planned_costs_in_buckets(self, reporting_period=None, buckets=None):
        """The function return the planned overall costs

        Args:
        no arguments

        Returns:
        planned costs (String)

        Raises:
        No exceptions planned"""
        planned_effort_accumulated = dict()
        planned_effort_accumulated['sum_costs'] = 0
        if buckets:
            for bucket in buckets:
                planned_effort_accumulated[bucket] = 0
        all_project_tasks = Task.objects.filter(project=self.id)
        if all_project_tasks:
            for task in all_project_tasks:
                planned_effort_accumulated_per_task = task.planned_costs_in_buckets(reporting_period, buckets)
                if buckets:
                    for bucket in buckets:
                        planned_effort_accumulated[bucket] += planned_effort_accumulated_per_task[bucket]
                planned_effort_accumulated['sum_costs'] += planned_effort_accumulated_per_task['sum_costs']

        if buckets:
            for bucket in buckets:
                planned_effort_accumulated[bucket] = Decimal(planned_effort_accumulated[bucket])
                self.default_currency.round(planned_effort_accumulated[bucket])
        planned_effort_accumulated['sum_costs'] = Decimal(planned_effort_accumulated['sum_costs'])
        self.default_currency.round(planned_effort_accumulated['sum_costs'])
        return planned_effort_accumulated

    def planned_costs(self, reporting_period=None, remaining=True):
        all_project_tasks = Task.objects.filter(project=self.id)
        planned_costs = 0
        if all_project_tasks:
            for task in all_project_tasks:
                planned_costs += task.planned_costs(reporting_period=reporting_period, remaining=remaining)
        return planned_costs

    def planned_total_costs(self):
        return self.planned_costs(remaining=False)
    planned_total_costs.short_description = _("Planned Total Costs")
    planned_total_costs.tags = True

    def effective_start(self):
        """The function return the effective start of a project as a date

        Args:
        no arguments

        Returns:
        effective_start (Date) or None when not yet started

        Raises:
        No exceptions planned"""
        no_tasks_started = True
        all_project_tasks = Task.objects.filter(project=self.id)
        effective_project_start = None
        if len(all_project_tasks) == 0:
            effective_project_start = None
        else:
            for task in all_project_tasks:
                if not effective_project_start:
                    if task.effective_start():
                        effective_project_start = task.effective_start()
                        no_tasks_started = False
                effective_task_start = task.effective_start()
                if effective_task_start:
                    if effective_task_start < effective_project_start:
                        effective_project_start = effective_task_start
            if no_tasks_started:
                effective_project_start = None
        return effective_project_start
    effective_start.short_description = _("Effective Start")
    effective_start.tags = True

    def effective_end(self):
        """The function return the effective end of a project as a date

        Args:
        no arguments

        Returns:
        effective_end (Date) or None when not yet ended

        Raises:
        No exceptions planned"""
        all_tasks_done = True
        all_project_tasks = Task.objects.filter(project=self.id)
        effective_project_end = None
        if len(all_project_tasks) == 0:
            effective_project_end = None
        else:
            i = 0
            for task in all_project_tasks:
                if not effective_project_end:
                    if not task.effective_start():
                        all_tasks_done = False
                        break
                    else:
                        effective_project_end = task.effective_start()
                effective_task_end = task.effective_end()
                if not effective_task_end:
                    all_tasks_done = False
                    break
                elif effective_task_end > effective_project_end:
                    effective_project_end = effective_task_end
                i = i+1
            if not all_tasks_done:
                effective_project_end = None
        return effective_project_end
    effective_end.short_description = _("Effective End")
    effective_end.tags = True

    def effective_duration(self):
        """The function return the effective overall duration of a project as a string in days
        The function is reading the effective_starts and effective_ends of the project and
        subtract them from each other.

        Args:
        no arguments

        Returns:
        duration_in_days or description (String)

        Raises:
        No exceptions planned"""
        effective_end = self.effective_end()
        effective_start = self.effective_start()
        if not effective_start:
            duration_as_string = "Project has not yet started"
        elif not effective_end:
            duration_as_string = "Project has not yet ended"
        else:
            duration_as_date = self.effective_end()-self.effective_start()
            duration_as_string = duration_as_date.days.__str__()
        return duration_as_string
    effective_duration.short_description = _("Effective Duration [dys]")
    effective_duration.tags = True

    def planned_start(self):
        """ The function return planned overall start of a project as a date
        the function finds all tasks within this project and finds the earliest start date.
        when no task is attached the task has which are attached have no start_date set, the
        function returns a None value

        Args:
        no arguments

        Returns:
        planned_end (datetime.Date) or  None

        Raises:
        No exceptions planned"""
        tasks = Task.objects.filter(project=self.id)
        if tasks:
            i = 0
            project_start = None
            for task in tasks:
                if task.planned_start():
                    if i == 0:
                        project_start = task.planned_start()
                    elif task.planned_start() < project_start:
                        project_start = task.planned_start()
                    i += 1
            return project_start
        else:
            return None

    def planned_end(self):
        """T he function return planned overall end of a project as a date
        the function finds all tasks within this project and finds the latest start end_date.
        when no task is attached the task has which are attached have no end_date set, the
        function returns a None value

        Args:
        no arguments

        Returns:
        planned_end (datetime.Date)

        Raises:
        No exceptions planned"""
        tasks = Task.objects.filter(project=self.id)
        if tasks:
            i = 0
            project_end = None
            for task in tasks:
                if task.planned_end():
                    if i == 0:
                        project_end = task.planned_end()
                    elif task.planned_end() > project_end:
                        project_end = task.planned_end()
                    i += 1
                    return project_end
        else:
            return None

    def planned_duration(self):
        """The function return planned overall duration of a project as a string in days

        Args:
        no arguments

        Returns:
        duration_in_days (String)

        Raises:
        No exceptions planned"""
        if (not self.planned_start()) or (not self.planned_end()):
            duration_in_days = "n/a"
        elif self.planned_start() > self.planned_end():
            duration_in_days = "n/a"
        else:
            duration_in_days = (self.planned_end()-self.planned_start()).days.__str__()
        return duration_in_days
    planned_duration.short_description = _("Planned Duration [dys]")
    planned_duration.tags = True

    def get_project_name(self):
        """The function safely returns a project_name even if the values is empty, in such a case
        the function returns a n/a"

        Args:
        no arguments

        Returns:
        project_name (String)

        Raises:
        No exceptions planned"""
        if self.project_name:
            return self.project_name
        else:
            return "n/a"

    def is_reporting_allowed(self):
        """The function returns a boolean True when it is allowed to report on the project and
        on one of the reporting periods does also allow reporting

        Args:
          no arguments

        Returns:
          reporting_allowed (Boolean)

        Raises:
          No exceptions planned"""
        from koalixcrm.crm.reporting.reporting_period import ReportingPeriod
        reporting_periods = ReportingPeriod.objects.filter(project=self.id, status__is_done=False)
        if len(reporting_periods) != 0:
            if not self.project_status.is_done:
                return True
            else:
                return False
        else:
            return False

    def __str__(self):
        return str(self.id)+" "+self.get_project_name()

    class Meta:
        app_label = "crm"
        verbose_name = _('Project')
        verbose_name_plural = _('Projects')


class ProjectAdminView(admin.ModelAdmin):
    list_display = ('id',
                    'project_name',
                    'project_manager',
                    'project_status',
                    'planned_total_costs',
                    'planned_duration',
                    'effective_duration',
                    'effective_effort',
                    'effective_costs_confirmed',
                    'effective_costs_not_confirmed'
                    )

    list_display_links = ('id',)
    ordering = ('-id',)

    fieldsets = (
        (_('Project'), {
            'fields': ('project_name',
                       'description',
                       'project_status',
                       'project_manager',
                       'default_currency',
                       'default_template_set',)
        }),
    )

    inlines = [TaskInlineAdminView,
               GenericLinkInlineAdminView,
               ReportingPeriodInlineAdminView]
    actions = ['create_report_pdf', ]

    def save_model(self, request, obj, form, change):
        if change:
            obj.last_modified_by = request.user
        else:
            obj.last_modified_by = request.user
            obj.staff = request.user
        obj.save()

    def create_report_pdf(self, request, queryset):
        from koalixcrm.crm.views.pdfexport import PDFExportView
        for obj in queryset:
            response = PDFExportView.export_pdf(self,
                                                request,
                                                obj,
                                                ("/admin/crm/"+obj.__class__.__name__.lower()+"/"),
                                                obj.default_template_set.monthly_project_summary_template)
            return response

    create_report_pdf.short_description = _("Create Report PDF")


class ProjectInlineAdminView(admin.TabularInline):
    model = Project
    readonly_fields = ('link_to_project',
                       'project_name',
                       'description',
                       'project_status',
                       'project_manager',
                       'default_template_set')
    fieldsets = (
        (_('Project'), {
            'fields': ('link_to_project',
                       'project_name',
                       'description',
                       'project_status',
                       'project_manager',
                       'default_template_set')
        }),
    )
    extra = 0

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False



# -*- coding: utf-8 -*-

from django.forms import NumberInput
from koalixcrm.crm.reporting.task import Task
from koalixcrm.crm.reporting.project import Project
from django.contrib.admin.widgets import *
from koalixcrm.global_support_functions import limit_string_length
from koalixcrm.djangoUserExtension.models import UserExtension
from koalixcrm.crm.reporting.reporting_period import ReportingPeriod
from koalixcrm.crm.reporting.human_resource import HumanResource


class WorkEntry(forms.Form):
    """
    Form which allows to fill out a full Work. Instead of only showing the task, it is possible
    to select the task based on the input from the project
    """
    project = forms.ModelChoiceField(queryset=Project.objects.filter(reportingperiod__status__is_done=False).distinct(),
                                     required=True)
    task = forms.ModelChoiceField(queryset=Task.objects.filter(status__is_done=False),
                                  required=True)
    datetime_start = forms.SplitDateTimeField(widget=AdminSplitDateTime,
                                              required=True)
    datetime_stop = forms.SplitDateTimeField(widget=AdminSplitDateTime,
                                             required=False)
    worked_hours = forms.DecimalField(widget=NumberInput(attrs={'step': 0.1,
                                                                'min': 0,
                                                                'max': 24}),
                                      required=False)
    description = forms.CharField(widget=AdminTextareaWidget,
                                  required=True)
    work_id = forms.IntegerField(widget=forms.HiddenInput(),
                                 required=False)

    def __init__(self, *args, **kwargs):
        self.from_date = kwargs.pop('from_date')
        self.to_date = kwargs.pop('to_date')
        self.original_from_date = self.from_date
        self.original_to_date = self.to_date
        super(WorkEntry, self).__init__(*args, **kwargs)

    @staticmethod
    def check_working_hours(cleaned_data):
        """This method checks that the working hour is correctly proved either using the start_stop pattern
        or by providing the worked_hours in total.

        Args:
          cleaned_data (Dict):  The cleaned_data must contain the values form the form validation.
          The django built in form validation must already have been passed

        Returns:
          True when no ValidationError was raised

        Raises:
          may raise ValidationError exception"""
        if ("datetime_start" in cleaned_data) & ("datetime_stop" in cleaned_data) & ("worked_hours" in cleaned_data):
            start_stop_pattern_complete = bool(cleaned_data["datetime_start"]) & bool(cleaned_data["datetime_stop"])
            start_stop_pattern_stop_missing = bool(cleaned_data["datetime_start"]) & (not bool(cleaned_data["datetime_stop"]))
            start_stop_pattern_start_missing = (not bool(cleaned_data["datetime_start"])) & bool(cleaned_data["datetime_stop"])
            worked_hours_pattern = bool(cleaned_data["worked_hours"])
        else:
            raise forms.ValidationError('Programming error', code='invalid')
        if start_stop_pattern_complete & worked_hours_pattern:
            raise forms.ValidationError('Please either set the start, stop time or worked hours (not both)',
                                        code='invalid')
        elif start_stop_pattern_start_missing or start_stop_pattern_stop_missing:
            raise forms.ValidationError('Set start and stop time',
                                        code='invalid')
        elif not start_stop_pattern_complete and not worked_hours_pattern:
            raise forms.ValidationError('Either fill out the start_time and stop_time or the worked_hours',
                                        code='invalid')
        return True

    def clean(self):
        cleaned_data = super(WorkEntry, self).clean()
        if 'date' in cleaned_data:
            date = cleaned_data['date']
            if date < self.from_date:
                raise forms.ValidationError('date is not within the selected range', code='invalid')
            elif self.to_date < date:
                raise forms.ValidationError('date is not within the selected range', code='invalid')
        if 'project' in cleaned_data:
            if not cleaned_data["project"].is_reporting_allowed():
                raise forms.ValidationError('The project is either closed or there is not '
                                            'reporting period available', code='invalid')
        WorkEntry.check_working_hours(cleaned_data)
        return cleaned_data

    def update_work(self, request):
        from koalixcrm.crm.reporting.work import Work
        if self.has_changed():
            if self.cleaned_data['work_id']:
                work = Work.objects.get(id=self.cleaned_data['work_id'])
            else:
                if not self.cleaned_data['DELETE']:
                    work = Work()
                else:
                    return
            if self.cleaned_data['DELETE']:
                work.delete()
            else:
                work.task = self.cleaned_data['task']
                work.reporting_period = ReportingPeriod.get_reporting_period(project=self.cleaned_data['task'].project,
                                                                             search_date=self.cleaned_data['datetime_start'].date())
                work.human_resource = HumanResource.objects.get(user=UserExtension.get_user_extension(request.user))
                work.date = self.cleaned_data['datetime_start'].date()
                if bool(self.cleaned_data['datetime_start']) & bool(self.cleaned_data['datetime_stop']):
                    work.start_time = self.cleaned_data['datetime_start']
                    work.stop_time = self.cleaned_data['datetime_stop']
                else:
                    work.worked_hours = self.cleaned_data['worked_hours']
                work.description = self.cleaned_data['description']
                work.short_description = limit_string_length(work.description, 100)
                work.save()



from django.contrib.auth.models import User
from rest_framework import serializers

from koalixcrm.crm.contact.customer import Customer
from koalixcrm.crm.contact.customer_billing_cycle import CustomerBillingCycle
from koalixcrm.crm.contact.customer_group import CustomerGroup
from koalixcrm.crm.rest.contact_rest import ContactJSONSerializer
from koalixcrm.crm.rest.customer_billing_cycle_rest import OptionCustomerBillingCycleJSONSerializer
from koalixcrm.crm.rest.customer_group_rest import OptionCustomerGroupJSONSerializer


class CustomerJSONSerializer(ContactJSONSerializer):
    defaultCustomerBillingCycle = OptionCustomerBillingCycleJSONSerializer(source='default_customer_billing_cycle')
    isMemberOf = OptionCustomerGroupJSONSerializer(source='is_member_of', many=True)
    isLead = serializers.BooleanField(source='is_lead')

    class Meta:
        model = Customer
        fields = ContactJSONSerializer.Meta.fields + (
            'id',
            'defaultCustomerBillingCycle',
            'isMemberOf',
            'isLead', )

    def create(self, validated_data):
        customer = Customer()
        customer.name = validated_data['name']
        customer.is_lead = validated_data['is_lead']

        # Deserialize from default billing cycle
        billing_cycle = validated_data.pop('default_customer_billing_cycle')
        if billing_cycle:
            if billing_cycle.get('id', None):
                customer.default_customer_billing_cycle = CustomerBillingCycle.objects.get(id=billing_cycle.get('id', None))
            else:
                customer.default_customer_billing_cycle = None

        # Deserialize from staff
        request = self.context.get('request')
        koalixcrm_user = request.META.get('HTTP_KOALIXCRM_USER')
        user = User.objects.get(username=koalixcrm_user)
        customer.last_modified_by = user

        customer.save()

        # Deserialize from customer group
        # Customer object has to be saved before to be able to add some many to many items. After adding a new many to
        # many item, the save action for the customer object is not needed since it's executed anyway.
        customer_groups = validated_data.pop('is_member_of')
        if customer_groups:
            for customerGroup in customer_groups:
                if customerGroup.get('id', None):
                    customer.is_member_of.add(CustomerGroup.objects.get(id=customerGroup.get('id', None)))

        return customer

    def update(self, customer, validated_data):
        customer.name = validated_data.get('name', customer.name)
        customer.is_lead = validated_data.get('is_lead', customer.is_lead)

        # Deserialize from staff
        request = self.context.get('request')
        koalixcrm_user = request.META.get('HTTP_KOALIXCRM_USER')
        user = User.objects.get(username=koalixcrm_user)
        customer.last_modified_by = user

        # Deserialize from customer group
        # Clear all from existing customer and add the deserialized items again.
        customer_groups = validated_data.pop('is_member_of')
        customer.is_member_of.clear()
        if customer_groups:
            for customerGroup in customer_groups:
                if customerGroup.get('id', None):
                    customer.is_member_of.add(CustomerGroup.objects.get(id=customerGroup.get('id', None)))

        customer.save()

        return customer



# -*- coding: utf-8 -*-
from rest_framework import serializers
from koalixcrm.crm.reporting.project import Project
from koalixcrm.crm.reporting.reporting_period import ReportingPeriod
from koalixcrm.crm.reporting.reporting_period_status import ReportingPeriodStatus
from koalixcrm.crm.rest.reporting_period_status_rest import OptionReportingPeriodStatusJSONSerializer
from koalixcrm.crm.rest.project_rest import OptionProjectJSONSerializer


class OptionReportingPeriodJSONSerializer(serializers.HyperlinkedModelSerializer):
    project = OptionProjectJSONSerializer(source='project', read_only=True)
    title = serializers.CharField(source='title', read_only=True)
    begin = serializers.DateField(source='begin', read_only=True)
    end = serializers.DateField(source='end', read_only=True)
    status = OptionReportingPeriodStatusJSONSerializer(source='status', read_only=True)

    class Meta:
        model = ReportingPeriod
        fields = ('project',
                  'title',
                  'begin',
                  'end',
                  'status')


class ReportingPeriodJSONSerializer(serializers.HyperlinkedModelSerializer):
    project = OptionProjectJSONSerializer(source='project')
    title = serializers.CharField(source='title')
    begin = serializers.DateField(source='begin')
    end = serializers.DateField(source='end')
    status = OptionReportingPeriodStatusJSONSerializer(source='status')

    class Meta:
        model = ReportingPeriod
        fields = ('project',
                  'title',
                  'begin',
                  'end',
                  'status')

    def create(self, validated_data):
        reporting_period = ReportingPeriod()
        # Deserialize project
        project = validated_data.pop('project')
        if project:
            if project.get('id', None):
                reporting_period.project = Project.objects.get(id=project.get('id', None))
            else:
                reporting_period.project = None
        # Deserialize status
        status = validated_data.pop('status')
        if status:
            if status.get('id', None):
                reporting_period.status = ReportingPeriodStatus.objects.get(id=status.get('id', None))
            else:
                reporting_period.status = None
        reporting_period.title = validated_data['title']
        reporting_period.begin = validated_data['begin']
        reporting_period.end = validated_data['end']
        reporting_period.save()
        return reporting_period

    def update(self, reporting_period, validated_data):
        # Deserialize project
        project = validated_data.pop('project')
        if project:
            if project.get('id', reporting_period.project):
                reporting_period.project = Project.objects.get(id=project.get('id', None))
            else:
                reporting_period.project = reporting_period.project_id
        else:
            reporting_period.project = None
        # Deserialize status
        status = validated_data.pop('status')
        if status:
            if status.get('id', reporting_period.status):
                reporting_period.status = ReportingPeriodStatus.objects.get(id=status.get('id', None))
            else:
                reporting_period.status = reporting_period.status_id
        else:
            reporting_period.status = None
        reporting_period.title = validated_data['title']
        reporting_period.begin = validated_data['begin']
        reporting_period.end = validated_data['end']
        reporting_period.save()
        return reporting_period






# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import GenericTaskLink
from koalixcrm.crm.factories.factory_task import StandardTaskFactory
from koalixcrm.crm.factories.factory_task_link_type import RelatedToTaskLinkTypeFactory
from koalixcrm.crm.factories.factory_user import StaffUserFactory
from koalixcrm.crm.factories.factory_project import StandardProjectFactory
from django.contrib.contenttypes.models import ContentType


class StandardGenericTaskLinkFactory(factory.django.DjangoModelFactory):
    class Meta:
        exclude = ['generic_crm_object']

    task = factory.SubFactory(StandardTaskFactory)
    task_link_type = factory.SubFactory(RelatedToTaskLinkTypeFactory)
    content_type = factory.LazyAttribute(
        lambda o: ContentType.objects.get_for_model(o.generic_crm_object))
    object_id = factory.SelfAttribute('generic_crm_object.id')
    date_of_creation = "2018-05-01"
    last_modified_by = factory.SubFactory(StaffUserFactory)


class LinkToProjectGenericTaskLinkFactory(StandardGenericTaskLinkFactory):
    class Meta:
        model = GenericTaskLink

    generic_crm_object = factory.SubFactory(StandardProjectFactory)



# -*- coding: utf-8 -*-

from rest_framework import viewsets
from rest_framework.authentication import BasicAuthentication, SessionAuthentication
from rest_framework.decorators import authentication_classes
from rest_framework.decorators import permission_classes
from rest_framework.permissions import IsAuthenticated
from rest_framework.renderers import JSONRenderer, BrowsableAPIRenderer
from rest_framework_xml.renderers import XMLRenderer

from koalixcrm.crm.contact.contact import PostalAddressForContact, EmailAddressForContact, PhoneAddressForContact
from koalixcrm.crm.contact.customer import Customer
from koalixcrm.crm.contact.customer_billing_cycle import CustomerBillingCycle
from koalixcrm.crm.contact.customer_group import CustomerGroup
from koalixcrm.crm.documents.contract import Contract, ContractJSONSerializer
from koalixcrm.crm.product.currency import Currency
from koalixcrm.crm.product.product_type import ProductType
from koalixcrm.crm.product.tax import Tax
from koalixcrm.crm.product.unit import Unit
from koalixcrm.crm.reporting.agreement import Agreement
from koalixcrm.crm.reporting.project import Project
from koalixcrm.crm.reporting.project_status import ProjectStatus
from koalixcrm.crm.reporting.task import Task
from koalixcrm.crm.reporting.task_status import TaskStatus
from koalixcrm.crm.rest.contact_rest import ContactPostalAddressJSONSerializer, ContactEmailAddressJSONSerializer, \
    ContactPhoneAddressJSONSerializer
from koalixcrm.crm.rest.currency_rest import CurrencyJSONSerializer
from koalixcrm.crm.rest.customer_billing_cycle_rest import CustomerBillingCycleJSONSerializer
from koalixcrm.crm.rest.customer_group_rest import CustomerGroupJSONSerializer
from koalixcrm.crm.rest.customer_rest import CustomerJSONSerializer
from koalixcrm.crm.rest.product_rest import ProductJSONSerializer
from koalixcrm.crm.rest.tax_rest import TaxJSONSerializer
from koalixcrm.crm.rest.unit_rest import UnitJSONSerializer
from koalixcrm.crm.rest.project_rest import ProjectJSONSerializer
from koalixcrm.crm.rest.project_status_rest import ProjectStatusJSONSerializer
from koalixcrm.crm.rest.task_rest import TaskJSONSerializer
from koalixcrm.crm.rest.task_status_rest import TaskStatusJSONSerializer
from koalixcrm.crm.rest.agreement_rest import AgreementJSONSerializer
from koalixcrm.crm.views.renderer import XSLFORenderer


class TaskAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows users to be viewed.
    """
    queryset = Task.objects.all()
    serializer_class = TaskJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer)
    filter_fields = ('project',)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(TaskAsJSON, self).dispatch(*args, **kwargs)


class ContractAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows users to be viewed.
    """
    queryset = Contract.objects.all()
    serializer_class = ContractJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(ContractAsJSON, self).dispatch(*args, **kwargs)


class TaskStatusAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows users to be viewed.
    """
    queryset = TaskStatus.objects.all()
    serializer_class = TaskStatusJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(TaskStatusAsJSON, self).dispatch(*args, **kwargs)


class CurrencyAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows currencies to be viewed.
    """
    queryset = Currency.objects.all()
    serializer_class = CurrencyJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(CurrencyAsJSON, self).dispatch(*args, **kwargs)


class CustomerBillingCycleAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows customer billing cycles to be viewed.
    """
    queryset = CustomerBillingCycle.objects.all()
    serializer_class = CustomerBillingCycleJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(CustomerBillingCycleAsJSON, self).dispatch(*args, **kwargs)


class CustomerAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows customers to be viewed.
    """
    queryset = Customer.objects.all()
    serializer_class = CustomerJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer, XSLFORenderer)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(CustomerAsJSON, self).dispatch(*args, **kwargs)


class CustomerGroupAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows customer groups to be viewed.
    """
    queryset = CustomerGroup.objects.all()
    serializer_class = CustomerGroupJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(CustomerGroupAsJSON, self).dispatch(*args, **kwargs)


class ContactEmailAddressAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows customer email addresses to be viewed.
    """
    queryset = EmailAddressForContact.objects.all()
    serializer_class = ContactEmailAddressJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer)
    filter_fields = ('person',)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(ContactEmailAddressAsJSON, self).dispatch(*args, **kwargs)


class ContactPhoneAddressAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows customer phone numbers to be viewed.
    """
    queryset = PhoneAddressForContact.objects.all()
    serializer_class = ContactPhoneAddressJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer)
    filter_fields = ('person',)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(ContactPhoneAddressAsJSON, self).dispatch(*args, **kwargs)


class ContactPostalAddressAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows customer postal addresses to be viewed.
    """
    queryset = PostalAddressForContact.objects.all()
    serializer_class = ContactPostalAddressJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer)
    filter_fields = ('person',)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(ContactPostalAddressAsJSON, self).dispatch(*args, **kwargs)


class TaxAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows taxes to be viewed.
    """
    queryset = Tax.objects.all()
    serializer_class = TaxJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(TaxAsJSON, self).dispatch(*args, **kwargs)


class UnitAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows units to be viewed.
    """
    queryset = Unit.objects.all()
    serializer_class = UnitJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(UnitAsJSON, self).dispatch(*args, **kwargs)


class ProductAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows products to be viewed.
    """
    queryset = ProductType.objects.all()
    serializer_class = ProductJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(ProductAsJSON, self).dispatch(*args, **kwargs)


class ProjectAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows users to be viewed.
    """
    queryset = Project.objects.all()
    serializer_class = ProjectJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer, XSLFORenderer)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(ProjectAsJSON, self).dispatch(*args, **kwargs)


class ProjectStatusAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows users to be viewed.
    """
    queryset = ProjectStatus.objects.all()
    serializer_class = ProjectStatusJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer, XSLFORenderer)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    @permission_classes((IsAuthenticated,))
    def dispatch(self, *args, **kwargs):
        return super(ProjectStatusAsJSON, self).dispatch(*args, **kwargs)


class AgreementAsJSON(viewsets.ModelViewSet):
    """
    API endpoint that allows users to be viewed.
    """
    queryset = Agreement.objects.all()
    serializer_class = AgreementJSONSerializer
    renderer_classes = (BrowsableAPIRenderer, JSONRenderer, XMLRenderer, XSLFORenderer)

    @authentication_classes((SessionAuthentication, BasicAuthentication))
    def dispatch(self, *args, **kwargs):
        return super(AgreementAsJSON, self).dispatch(*args, **kwargs)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-07-07 12:40
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('accounting', '0002_auto_20170705_1702'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='account',
            name='originalAmount',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-06-12 19:24
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangoUserExtension', '0004_auto_20171210_2126'),
    ]

    operations = [
        migrations.CreateModel(
            name='MonthlyProjectSummaryTemplate',
            fields=[
                ('documenttemplate_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='djangoUserExtension.DocumentTemplate')),
            ],
            options={
                'verbose_name': 'Monthly project summary template',
                'verbose_name_plural': 'Monthly project summary templates',
            },
            bases=('djangoUserExtension.documenttemplate',),
        ),
        migrations.AddField(
            model_name='templateset',
            name='monthly_project_summary_template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='djangoUserExtension.MonthlyProjectSummaryTemplate'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-12 20:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0018_auto_20180111_2031'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='emailaddressforsalesdocument',
            options={'verbose_name': 'Email Address For Sales Documents', 'verbose_name_plural': 'Email Address For Sales Documents'},
        ),
        migrations.AlterModelOptions(
            name='phoneaddressforsalesdocument',
            options={'verbose_name': 'Phone Address For Sales Documents', 'verbose_name_plural': 'Phone Address For Sales Documents'},
        ),
        migrations.AlterModelOptions(
            name='postaladdressforsalesdocument',
            options={'verbose_name': 'Postal Address For Sales Documents', 'verbose_name_plural': 'Postal Address For Sales Documents'},
        ),
        migrations.RenameField(
            model_name='customerbillingcycle',
            old_name='timeToPaymentDate',
            new_name='time_to_payment_date',
        ),
        migrations.RemoveField(
            model_name='deliverynote',
            name='derived_from_invoice',
        ),
        migrations.RemoveField(
            model_name='deliverynote',
            name='derived_from_quote',
        ),
        migrations.RemoveField(
            model_name='paymentreminder',
            name='derived_from_invoice',
        ),
        migrations.RemoveField(
            model_name='purchaseconfirmation',
            name='derived_from_quote',
        ),
        migrations.AddField(
            model_name='customerbillingcycle',
            name='payment_reminder_time_to_payment',
            field=models.IntegerField(default=10, verbose_name='Payment Reminder, Days To Payment Date '),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-08 20:48
from __future__ import unicode_literals
from django.db import migrations
from django.db.migrations.operations.base import Operation
from django.db.models.fields.related import RECURSIVE_RELATIONSHIP_CONSTANT


class MigrateSalesContractToSalesDocument(Operation):

    # If this is False, it means that this operation will be ignored by
    # sqlmigrate; if true, it will be run and the SQL collected for its output.
    reduces_to_sql = False

    # If this is False, Django will refuse to reverse past this operation.
    reversible = False

    def __init__(self, arg1, arg2):
        # Operations are usually instantiated with arguments in migration
        # files. Store the values of them on self for later use.
        pass


    def _get_model_tuple(self, remote_model, app_label, model_name):
        if remote_model == RECURSIVE_RELATIONSHIP_CONSTANT:
            return app_label, model_name.lower()
        elif '.' in remote_model:
            return tuple(remote_model.lower().split('.'))
        else:
            return app_label, remote_model.lower()

    def state_forwards(self, app_label, state):
        # Add a new model.
        renamed_model = state.models["crm", "salescontract"].clone()
        renamed_model.name = "SalesDocument"
        state.models["crm", "salesdocument"] = renamed_model
        # Repoint all fields pointing to the old model to the new one.
        old_model_tuple = "crm", "salescontract"
        old_remote_model = '%s.%s' % ("crm", "salescontract")
        new_remote_model = '%s.%s' % ("crm", "SalesDocument")
        to_reload = []
        for (model_app_label, model_name), model_state in state.models.items():
            if model_name != "salescontract" and model_name != "salesdocument":
                new_model_state = model_state
                model_with_new_base = state.models[model_app_label, model_name]
                for index, (name, field) in enumerate(model_state.fields):
                    changed_field = None
                    remote_field = field.remote_field
                    if remote_field:
                        remote_model_tuple = self._get_model_tuple(
                            remote_field.model, model_app_label, model_name
                        )
                        if remote_model_tuple == old_model_tuple:
                            changed_field = field.clone()

                            changed_field.remote_field.model = new_remote_model
                        through_model = getattr(remote_field, 'through', None)
                        if through_model:
                            through_model_tuple = self._get_model_tuple(
                                through_model, model_app_label, model_name
                            )
                            if through_model_tuple == old_model_tuple:
                                if changed_field is None:
                                    changed_field = field.clone()
                                changed_field.remote_field.through = new_remote_model
                    if changed_field:
                        new_model_state = model_state.clone()
                        new_model_state.fields[index] = name, changed_field
                        model_changed = True
                        if old_remote_model in model_state.bases:
                            new_bases = []
                            for base in model_state.bases:
                                if old_remote_model == base:
                                    new_bases.append(new_remote_model.lower())
                                    new_model_state.bases = tuple(new_bases)
                        if model_changed:
                            state.remove_model(model_app_label, model_name)
                            state.models[model_app_label, model_name] = new_model_state
                            to_reload.append((model_app_label, model_name))
        # Reload models related to old model before removing the old model.
        state.remove_model("crm", "salescontract")
        state.reload_model("crm", "salesdocument", delay=True)
        state.reload_models(to_reload, delay=True)
        # Remove the old model.

    def database_forwards(self, app_label, schema_editor, from_state, to_state):
        # The Operation should use schema_editor to apply any changes it
        # wants to make to the database.
        new_model = to_state.apps.get_model("crm", "SalesDocument")
        if self.allow_migrate_model(schema_editor.connection.alias, new_model):
            old_model = from_state.apps.get_model("crm", "SalesContract")
            # Move the main table
            schema_editor.alter_db_table(
                new_model,
                old_model._meta.db_table,
                new_model._meta.db_table,
            )
            # Alter the fields pointing to us
            for related_object in old_model._meta.related_objects:
                if related_object.related_model == old_model:
                    model = new_model
                    related_key = ("crm", "salesdocument")
                else:
                    model = related_object.related_model
                    related_key = (
                        related_object.related_model._meta.app_label,
                        related_object.related_model._meta.model_name,
                    )
                to_field = to_state.apps.get_model(*related_key)._meta.get_field(related_object.field.name)

                schema_editor.alter_field(
                    model,
                    related_object.field,
                    to_field,
                )
            # Rename M2M fields whose name is based on this model's name.
            fields = zip(old_model._meta.local_many_to_many, new_model._meta.local_many_to_many)
            for (old_field, new_field) in fields:
                # Skip self-referential fields as these are renamed above.
                if new_field.model == new_field.related_model or not new_field.remote_field.through._meta.auto_created:
                    continue
                # Rename the M2M table that's based on this model's name.
                old_m2m_model = old_field.remote_field.through
                new_m2m_model = new_field.remote_field.through
                schema_editor.alter_db_table(
                    new_m2m_model,
                    old_m2m_model._meta.db_table,
                    new_m2m_model._meta.db_table,
                )
                # Rename the column in the M2M table that's based on this
                # model's name.
                schema_editor.alter_field(
                    new_m2m_model,
                    old_m2m_model._meta.get_field(old_model._meta.model_name),
                    new_m2m_model._meta.get_field(new_model._meta.model_name),
                )

    def database_backwards(self, app_label, schema_editor, from_state, to_state):
        # If reversible is True, this is called when the operation is reversed.
        pass

    def describe(self):
        # This is used to describe what the operation does in console output.
        return "Custom Operation"


class Migration(migrations.Migration):
    atomic = False
    dependencies = [
        ('crm', '0014_auto_20180108_2048'),
    ]

    operations = [

        migrations.RemoveField(
            model_name='salescontract',
            name='derived_from_sales_contract',
        ),
        MigrateSalesContractToSalesDocument("crm", "salescontract")
    ]



# -*- coding: utf-8 -*-
from koalixcrm.accounting.rest.account_rest import OptionAccountJSONSerializer
from rest_framework import serializers

from koalixcrm.accounting.accounting.product_category import ProductCategory
from koalixcrm.accounting.models import Account


class ProductCategoryMinimalJSONSerializer(serializers.HyperlinkedModelSerializer):
    id = serializers.IntegerField(required=False)
    title = serializers.CharField(read_only=True)

    class Meta:
        model = Account
        fields = ('id',
                  'title')


class ProductCategoryJSONSerializer(serializers.HyperlinkedModelSerializer):
    profitAccount = OptionAccountJSONSerializer(source='profit_account')
    lossAccount = OptionAccountJSONSerializer(source='loss_account')

    class Meta:
        model = ProductCategory
        fields = ('id',
                  'title',
                  'profitAccount',
                  'lossAccount')
        depth = 1

    def create(self, validated_data):
        product_category = ProductCategory()
        product_category.title = validated_data['title']

        # Deserialize profit account
        profit_account = validated_data.pop('profit_account')
        if profit_account:
            if profit_account.get('id', None):
                product_category.profit_account = Account.objects.get(id=profit_account.get('id', None))
            else:
                product_category.profit_account = None

        # Deserialize loss account
        loss_account = validated_data.pop('loss_account')
        if loss_account:
            if loss_account.get('id', None):
                product_category.loss_account = Account.objects.get(id=loss_account.get('id', None))
            else:
                product_category.loss_account = None

        product_category.save()
        return product_category

    def update(self, instance, validated_data):
        instance.title = validated_data.get('title', instance.title)

        # Deserialize profit account
        profit_account = validated_data.pop('profit_account')
        if profit_account:
            if profit_account.get('id', instance.profit_account_id):
                instance.profit_account = Account.objects.get(id=profit_account.get('id', None))
            else:
                instance.profit_account = instance.profit_account_id
        else:
            instance.profit_account = None

        # Deserialize loss account
        loss_account = validated_data.pop('loss_account')
        if loss_account:
            if loss_account.get('id', instance.loss_account_id):
                instance.loss_account = Account.objects.get(id=loss_account.get('id', None))
            else:
                instance.loss_account = instance.loss_account_id
        else:
            instance.loss_account = None

        instance.save()
        return instance



# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2018-01-11 20:11
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('crm', '0015_auto_20180111_2043'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='EmailAddressForSalesContract',
            new_name='EmailAddressForSalesDocument'
        ),
        migrations.RenameModel(
            old_name='PhoneAddressForSalesContract',
            new_name='PhoneAddressForSalesDocument',
        ),
        migrations.RenameModel(
            old_name='PostalAddressForSalesContract',
            new_name='PostalAddressForSalesDocument',
        ),
        migrations.RenameModel(
            old_name='SalesContractPosition',
            new_name='SalesDocumentPosition',
        ),
        migrations.RenameModel(
            old_name='TextParagraphInSalesContract',
            new_name='TextParagraphInSalesDocument',
        ),
        migrations.AddField(
            model_name='salesdocument',
            name='derived_from_sales_document',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='crm.SalesDocument'),
        ),
    ]



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import ReportingPeriodStatus


class InPreparationReportingPeriodStatusFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ReportingPeriodStatus
        django_get_or_create = ('title',)

    title = "In Preparation"
    description = "In Preparation, reporting not yet enabled"
    is_done = False


class ReportingReportingPeriodStatusFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ReportingPeriodStatus
        django_get_or_create = ('title',)

    title = "Reporting"
    description = "Reporting is enabled"
    is_done = False


class DoneReportingPeriodStatusFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = ReportingPeriodStatus
        django_get_or_create = ('title',)

    title = "Done"
    description = "This reporting period is done"
    is_done = True



from .base_settings import *

# Database
# https://docs.djangoproject.com/en/1.11/ref/settings/#databases
ALLOWED_HOSTS = ['*']

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql',
        'NAME': 'postgres',
        'USER': 'postgres',
        'HOST': 'db',
        'PORT': 5432,
    }
}

FOP_EXECUTABLE = "/usr/bin/fop-2.2/fop/fop"
GRAPPELLI_INDEX_DASHBOARD = 'projectsettings.dashboard.CustomIndexDashboard'
FILEBROWSER_CONVERT_FILENAME = False
KOALIXCRM_REST_API_AUTH = False


# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import Project
from koalixcrm.crm.factories.factory_project_status import StartedProjectStatusFactory
from koalixcrm.djangoUserExtension.factories.factory_template_set import StandardTemplateSetFactory
from koalixcrm.crm.factories.factory_user import StaffUserFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory


class StandardProjectFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Project
        django_get_or_create = ('project_name',)

    project_manager = factory.SubFactory(StaffUserFactory)
    project_name = "This is a Test Project"
    description = "This is description of a test Project"
    project_status = factory.SubFactory(StartedProjectStatusFactory)
    default_template_set = factory.SubFactory(StandardTemplateSetFactory)
    default_currency = factory.SubFactory(StandardCurrencyFactory)
    date_of_creation = "2018-05-01"
    last_modification = "2018-05-02"
    last_modified_by = factory.SubFactory(StaffUserFactory)



# -*- coding: utf-8 -*-


from koalixcrm.djangoUserExtension.models import *
from koalixcrm.djangoUserExtension.user_extension.user_extension import UserExtension, OptionUserExtension
from koalixcrm.djangoUserExtension.user_extension.template_set import TemplateSet, OptionTemplateSet
from koalixcrm.djangoUserExtension.user_extension.document_template import InvoiceTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import QuoteTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import DeliveryNoteTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import PaymentReminderTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import PurchaseOrderTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import ProfitLossStatementTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import BalanceSheetTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import OptionDocumentTemplate


admin.site.register(UserExtension, OptionUserExtension)
admin.site.register(TemplateSet, OptionTemplateSet)
admin.site.register(InvoiceTemplate, OptionDocumentTemplate)
admin.site.register(QuoteTemplate, OptionDocumentTemplate)
admin.site.register(DeliveryNoteTemplate, OptionDocumentTemplate)
admin.site.register(PaymentReminderTemplate, OptionDocumentTemplate)
admin.site.register(PurchaseOrderTemplate, OptionDocumentTemplate)
admin.site.register(PurchaseConfirmationTemplate, OptionDocumentTemplate)
admin.site.register(ProfitLossStatementTemplate, OptionDocumentTemplate)
admin.site.register(BalanceSheetTemplate, OptionDocumentTemplate)
admin.site.register(MonthlyProjectSummaryTemplate, OptionDocumentTemplate)
admin.site.register(WorkReportTemplate, OptionDocumentTemplate)



# -*- coding: utf-8 -*-

import os
from subprocess import check_output
from subprocess import STDOUT
from django.conf import settings
from django.core import serializers
from lxml import etree
import koalixcrm.crm.documents.sales_document
import koalixcrm.djangoUserExtension.models


class PDFExport:

    @staticmethod
    def find_element_in_xml(xml_string, find_pattern, find_value):
        parser = etree.XMLParser(encoding='utf-8', remove_blank_text=True)
        root_element = etree.fromstring(xml_string.encode('utf-8'), parser=parser)
        found_element = root_element.findall(find_pattern)
        if found_element is None:
            return 0
        else:
            for element in found_element:
                if element.text == find_value:
                    return 1
            return 0

    @staticmethod
    def append_element_to_pattern(xml_string, find_pattern, name_of_element, value_of_element, **kwargs):
        attributes = kwargs.get('attributes', None)
        parser = etree.XMLParser(encoding='utf-8', remove_blank_text=True)
        root_element = etree.fromstring(xml_string.encode('utf-8'), parser=parser)
        found_element = root_element.find(find_pattern)
        new_element = etree.SubElement(found_element, name_of_element, attrib=attributes)
        new_element.text = value_of_element.__str__()
        return (etree.tostring(root_element,
                               encoding='UTF-8',
                               xml_declaration=True,
                               pretty_print=True)).decode('utf-8')

    @staticmethod
    def merge_xml(xml_string_1, xml_string_2):
        parser = etree.XMLParser(encoding='utf-8', remove_blank_text=True)
        root_element_1 = etree.fromstring(xml_string_1.encode('utf-8'), parser=parser)
        root_element_2 = etree.fromstring(xml_string_2.encode('utf-8'), parser=parser)
        for child in root_element_2:
            root_element_1.append(child)
        return (etree.tostring(root_element_1,
                               encoding='UTF-8',
                               xml_declaration=True,
                               pretty_print=True)).decode('utf-8')

    @staticmethod
    def write_xml(objects_to_serialize):
        xml = serializers.serialize("xml", objects_to_serialize, indent=3)
        return xml

    @staticmethod
    def write_xml_file(xml, file_path):
        f = open(file_path, "wb+")
        f.truncate()
        f.write(xml.encode('utf-8'))
        f.close()

    @staticmethod
    def perform_xsl_transformation(file_with_serialized_xml, xsl_file, fop_config_file, file_output_pdf):
        check_output([settings.FOP_EXECUTABLE,
                      '-c', fop_config_file.path_full,
                      '-xml', os.path.join(settings.PDF_OUTPUT_ROOT, file_with_serialized_xml),
                      '-xsl', xsl_file.path_full,
                      '-pdf', file_output_pdf], stderr=STDOUT)

    @staticmethod
    def create_pdf(object_to_create_pdf, template_set, printed_by, *args, **kwargs):
        # define the files which are involved in pdf creation process
        fop_config_file = object_to_create_pdf.get_fop_config_file(template_set)
        xsl_file = object_to_create_pdf.get_xsl_file(template_set)
        file_with_serialized_xml = os.path.join(settings.PDF_OUTPUT_ROOT, (str(type(object_to_create_pdf).__name__) +
                                                                           "_" + str(object_to_create_pdf.id) + ".xml"))
        file_output_pdf = os.path.join(settings.PDF_OUTPUT_ROOT, (str(type(object_to_create_pdf).__name__) +
                                                                  "_" + str(object_to_create_pdf.id) + ".pdf"))

        # list the sub-objects which have to be serialized
        xml_string = object_to_create_pdf.serialize_to_xml(*args, **kwargs)
        objects_to_serialize = list(koalixcrm.djangoUserExtension.models.DocumentTemplate.objects.filter(id=template_set.id))
        xml_string_temp = PDFExport.write_xml(objects_to_serialize)
        xml_string = PDFExport.merge_xml(xml_string, xml_string_temp)
        objects_to_serialize = koalixcrm.djangoUserExtension.models.UserExtension.objects_to_serialize(object_to_create_pdf, printed_by)
        xml_string_temp = PDFExport.write_xml(objects_to_serialize)
        xml_string = PDFExport.merge_xml(xml_string, xml_string_temp)

        # extend the xml-string with required basic settings
        xml_string = PDFExport.append_element_to_pattern(xml_string,
                                                         ".",
                                                         "filebrowser_directory",
                                                         settings.MEDIA_ROOT)

        #  write xml-string to xml-file
        PDFExport.write_xml_file(xml_string, file_with_serialized_xml)

        # perform xsl transformation
        PDFExport.perform_xsl_transformation(file_with_serialized_xml, xsl_file, fop_config_file, file_output_pdf)

        return file_output_pdf



# -*- coding: utf-8 -*-

import factory
import datetime
from koalixcrm.global_support_functions import make_date_utc
from koalixcrm.crm.models import SalesDocument
from koalixcrm.crm.factories.factory_user import StaffUserFactory
from koalixcrm.crm.factories.factory_contract import StandardContractFactory
from koalixcrm.crm.factories.factory_customer import StandardCustomerFactory
from koalixcrm.crm.factories.factory_currency import StandardCurrencyFactory
from koalixcrm.djangoUserExtension.factories.factory_document_template import StandardQuoteTemplateFactory


class StandardSalesDocumentFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = SalesDocument

    contract = factory.SubFactory(StandardContractFactory)
    external_reference = "This is an external Reference"
    discount = "0"
    description = "This is the description of a sales document"
    last_pricing_date = make_date_utc(datetime.datetime(2018, 5, 1, 00))
    last_calculated_price = "220.00"
    last_calculated_tax = "10.00"
    customer = factory.SubFactory(StandardCustomerFactory)
    staff = factory.SubFactory(StaffUserFactory)
    currency = factory.SubFactory(StandardCurrencyFactory)
    date_of_creation = make_date_utc(datetime.datetime(2018, 5, 1, 00))
    custom_date_field = make_date_utc(datetime.datetime(2018, 5, 20, 00))
    last_modification = make_date_utc(datetime.datetime(2018, 5, 25, 00))
    last_modified_by = factory.SubFactory(StaffUserFactory)
    template_set = factory.SubFactory(StandardQuoteTemplateFactory)
    derived_from_sales_document = None
    last_print_date = make_date_utc(datetime.datetime(2018, 5, 26, 00))




from rest_framework import serializers

from koalixcrm.djangoUserExtension.user_extension.template_set import TemplateSet
from koalixcrm.djangoUserExtension.rest.document_template_rest import OptionInvoiceTemplateJSONSerializer
from koalixcrm.djangoUserExtension.rest.document_template_rest import OptionQuoteTemplateJSONSerializer
from koalixcrm.djangoUserExtension.rest.document_template_rest import OptionDeliveryNoteTemplateJSONSerializer
from koalixcrm.djangoUserExtension.rest.document_template_rest import OptionPaymentReminderTemplateJSONSerializer
from koalixcrm.djangoUserExtension.rest.document_template_rest import OptionPurchaseConfirmationTemplateJSONSerializer
from koalixcrm.djangoUserExtension.rest.document_template_rest import OptionProfitLossStatementTemplateJSONSerializer
from koalixcrm.djangoUserExtension.rest.document_template_rest import OptionPurchaseOrderTemplateJSONSerializer
from koalixcrm.djangoUserExtension.rest.document_template_rest import OptionBalanceSheetTemplateJSONSerializer
from koalixcrm.djangoUserExtension.rest.document_template_rest import OptionMonthlyProjectSummaryTemplateTemplateJSONSerializer
from koalixcrm.djangoUserExtension.rest.document_template_rest import OptionWorkReportTemplateJSONSerializer
from koalixcrm.djangoUserExtension.user_extension.document_template import InvoiceTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import QuoteTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import DeliveryNoteTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import PaymentReminderTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import PurchaseOrderTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import PurchaseConfirmationTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import ProfitLossStatementTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import BalanceSheetTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import MonthlyProjectSummaryTemplate
from koalixcrm.djangoUserExtension.user_extension.document_template import WorkReportTemplate


class OptionTemplateSetJSONSerializer(serializers.HyperlinkedModelSerializer):
    invoiceTemplate = OptionInvoiceTemplateJSONSerializer(source='invoice_template', read_only=True)
    quoteTemplate = OptionQuoteTemplateJSONSerializer(source='quote_template', read_only=True)
    deliveryNoteTemplate = OptionDeliveryNoteTemplateJSONSerializer(source='delivery_note_template', read_only=True)
    paymentReminderTemplate = OptionPaymentReminderTemplateJSONSerializer(source='purchase_confirmation_template', read_only=True)
    purchaseConfirmation_template = OptionPurchaseConfirmationTemplateJSONSerializer(source='user', read_only=True)
    purchaseOrderTemplate = OptionPurchaseOrderTemplateJSONSerializer(source='purchase_order_template', read_only=True)
    profitLossStatementTemplate = OptionProfitLossStatementTemplateJSONSerializer(source='profit_loss_statement_template', read_only=True)
    balanceSheetStatementTemplate = OptionBalanceSheetTemplateJSONSerializer(source='balance_sheet_statement_template', read_only=True)
    monthlyProjectSummaryTemplate = OptionMonthlyProjectSummaryTemplateTemplateJSONSerializer(source='monthly_project_summary_template', read_only=True)
    workReportTemplate = OptionWorkReportTemplateJSONSerializer(source='work_report_template', read_only=True)

    class Meta:
        model = TemplateSet
        fields = ('title',
                  'invoiceTemplate',
                  'quoteTemplate',
                  'deliveryNoteTemplate',
                  'paymentReminderTemplate',
                  'purchaseConfirmation_template',
                  'purchaseOrderTemplate',
                  'profitLossStatementTemplate',
                  'balanceSheetStatementTemplate',
                  'monthlyProjectSummaryTemplate',
                  'workReportTemplate')


class TemplateSetJSONSerializer(serializers.HyperlinkedModelSerializer):
    invoiceTemplate = OptionInvoiceTemplateJSONSerializer(source='invoice_template')
    quoteTemplate = OptionQuoteTemplateJSONSerializer(source='quote_template')
    deliveryNoteTemplate = OptionDeliveryNoteTemplateJSONSerializer(source='delivery_note_template')
    paymentReminderTemplate = OptionPaymentReminderTemplateJSONSerializer(source='purchase_confirmation_template')
    purchaseOrderTemplate = OptionPurchaseConfirmationTemplateJSONSerializer(source='user')
    purchaseOrderTemplate = OptionPurchaseOrderTemplateJSONSerializer(source='purchase_order_template')
    profitLossStatementTemplate = OptionProfitLossStatementTemplateJSONSerializer(source='profit_loss_statement_template')
    balanceSheetStatementTemplate = OptionBalanceSheetTemplateJSONSerializer(source='balance_sheet_statement_template')
    monthlyProjectSummaryTemplate = OptionMonthlyProjectSummaryTemplateTemplateJSONSerializer(source='monthly_project_summary_template')
    workReportTemplate = OptionWorkReportTemplateJSONSerializer(source='work_report_template')

    class Meta:
        model = TemplateSet
        fields = ('title',
                  'invoiceTemplate',
                  'quoteTemplate',
                  'deliveryNoteTemplate',
                  'paymentReminderTemplate',
                  'purchaseConfirmation_template',
                  'purchaseOrderTemplate',
                  'profitLossStatementTemplate',
                  'balanceSheetStatementTemplate',
                  'monthlyProjectSummaryTemplate',
                  'workReportTemplate')

    def create(self, validated_data):
        template_set = TemplateSet()
        # Deserialize invoice template
        invoice_template = validated_data.pop('invoiceTemplate')
        if invoice_template:
            if invoice_template.get('id', None):
                template_set.invoice_template = InvoiceTemplate.objects.get(id=invoice_template.get('id', None))
            else:
                template_set.invoice_template = None
        # Deserialize quote template
        quote_template = validated_data.pop('quoteTemplate')
        if quote_template:
            if quote_template.get('id', None):
                template_set.quote_template = QuoteTemplate.objects.get(id=quote_template.get('id', None))
            else:
                template_set.quote_template = None
        # Deserialize delivery note template
        delivery_note_template = validated_data.pop('deliveryNoteTemplate')
        if delivery_note_template:
            if delivery_note_template.get('id', None):
                template_set.delivery_note_template = DeliveryNoteTemplate.objects.get(id=delivery_note_template.get('id', None))
            else:
                template_set.delivery_note_template = None
        # Deserialize payment reminder template
        payment_reminder_template = validated_data.pop('paymentReminderTemplate')
        if payment_reminder_template:
            if payment_reminder_template.get('id', None):
                template_set.payment_reminder_template = PaymentReminderTemplate.objects.get(id=payment_reminder_template.get('id', None))
            else:
                template_set.payment_reminder_template = None
        # Deserialize purchase_confirmation_template
        purchase_confirmation_template = validated_data.pop('purchaseConfirmationTemplate')
        if purchase_confirmation_template:
            if purchase_confirmation_template.get('id', None):
                template_set.purchase_confirmation_template = PurchaseConfirmationTemplate.objects.get(id=purchase_confirmation_template.get('id', None))
            else:
                template_set.purchase_confirmation_template = None
        # Deserialize purchase_order_template
        purchase_order_template = validated_data.pop('purchaseOrderTemplate')
        if purchase_order_template:
            if purchase_order_template.get('id', None):
                template_set.purchase_order_template = PurchaseOrderTemplate.objects.get(id=purchase_order_template.get('id', None))
            else:
                template_set.purchase_order_template = None
        # Deserialize purchase_confirmation_template
        purchase_confirmation_template = validated_data.pop('purchaseConfirmationTemplate')
        if purchase_confirmation_template:
            if purchase_confirmation_template.get('id', None):
                template_set.purchase_confirmation_template = PurchaseConfirmationTemplate.objects.get(id=purchase_confirmation_template.get('id', None))
            else:
                template_set.purchase_confirmation_template = None
        # Deserialize profit_loss_statement_template
        profit_loss_statement_template = validated_data.pop('profitLossStatementTemplate')
        if profit_loss_statement_template:
            if profit_loss_statement_template.get('id', None):
                template_set.profit_loss_statement_template = ProfitLossStatementTemplate.objects.get(id=profit_loss_statement_template.get('id', None))
            else:
                template_set.profit_loss_statement_template = None
        # Deserialize balance_sheet_statement_template
        balance_sheet_statement_template = validated_data.pop('balanceSheetStatementTemplate')
        if balance_sheet_statement_template:
            if balance_sheet_statement_template.get('id', None):
                template_set.balance_sheet_statement_template = BalanceSheetTemplate.objects.get(id=balance_sheet_statement_template.get('id', None))
            else:
                template_set.balance_sheet_statement_template = None
        # Deserialize monthly_project_summary_template
        monthly_project_summary_template = validated_data.pop('monthlyProjectSummaryTemplate')
        if monthly_project_summary_template:
            if monthly_project_summary_template.get('id', None):
                template_set.monthly_project_summary_template = MonthlyProjectSummaryTemplate.objects.get(id=monthly_project_summary_template.get('id', None))
            else:
                template_set.monthly_project_summary_template = None
        # Deserialize work_report_template
        work_report_template = validated_data.pop('workReportTemplate')
        if work_report_template:
            if work_report_template.get('id', None):
                template_set.work_report_template = WorkReportTemplate.objects.get(id=work_report_template.get('id', None))
            else:
                template_set.work_report_template = None
        template_set.save()

    def update(self, template_set, validated_data):
        # Deserialize invoice template
        invoice_template = validated_data.pop('invoiceTemplate')
        if invoice_template:
            if invoice_template.get('id', None):
                template_set.invoice_template = InvoiceTemplate.objects.get(id=invoice_template.get('id', None))
            else:
                template_set.invoice_template = template_set.invoice_template_id
        else:
            template_set.invoice_template = None
        # Deserialize quote template
        quote_template = validated_data.pop('quoteTemplate')
        if quote_template:
            if quote_template.get('id', None):
                template_set.quote_template = QuoteTemplate.objects.get(id=quote_template.get('id', None))
            else:
                template_set.quote_template = template_set.quote_template_id
        else:
            template_set.quote_template = None
        # Deserialize delivery note template
        delivery_note_template = validated_data.pop('deliveryNoteTemplate')
        if delivery_note_template:
            if delivery_note_template.get('id', None):
                template_set.delivery_note_template = DeliveryNoteTemplate.objects.get(id=delivery_note_template.get('id', None))
            else:
                template_set.delivery_note_template = template_set.delivery_note_template_id
        else:
            template_set.delivery_note_template = None
        # Deserialize payment reminder template
        payment_reminder_template = validated_data.pop('paymentReminderTemplate')
        if payment_reminder_template:
            if payment_reminder_template.get('id', None):
                template_set.payment_reminder_template = PaymentReminderTemplate.objects.get(id=payment_reminder_template.get('id', None))
            else:
                template_set.payment_reminder_template = template_set.payment_reminder_template_id
        else:
            template_set.payment_reminder_template = None
        # Deserialize purchase_confirmation_template
        purchase_confirmation_template = validated_data.pop('purchaseConfirmationTemplate')
        if purchase_confirmation_template:
            if purchase_confirmation_template.get('id', None):
                template_set.purchase_confirmation_template = PurchaseConfirmationTemplate.objects.get(id=purchase_confirmation_template.get('id', None))
            else:
                template_set.purchase_confirmation_template = template_set.purchase_confirmation_template_id
        else:
            template_set.purchase_confirmation_template = None
        # Deserialize purchase_order_template
        purchase_order_template = validated_data.pop('purchaseOrderTemplate')
        if purchase_order_template:
            if purchase_order_template.get('id', None):
                template_set.purchase_order_template = PurchaseOrderTemplate.objects.get(id=purchase_order_template.get('id', None))
            else:
                template_set.purchase_order_template = template_set.purchase_order_template_id
        else:
            template_set.work_report_template = None
        # Deserialize purchase_confirmation_template
        purchase_confirmation_template = validated_data.pop('purchaseConfirmationTemplate')
        if purchase_confirmation_template:
            if purchase_confirmation_template.get('id', None):
                template_set.purchase_confirmation_template = PurchaseConfirmationTemplate.objects.get(id=purchase_confirmation_template.get('id', None))
            else:
                template_set.purchase_confirmation_template = template_set.purchase_confirmation_template_id
        else:
            template_set.purchase_confirmation_template = None
        # Deserialize profit_loss_statement_template
        profit_loss_statement_template = validated_data.pop('profitLossStatementTemplate')
        if profit_loss_statement_template:
            if profit_loss_statement_template.get('id', None):
                template_set.profit_loss_statement_template = ProfitLossStatementTemplate.objects.get(id=profit_loss_statement_template.get('id', None))
            else:
                template_set.profit_loss_statement_template = template_set.profit_loss_statement_template_id
        else:
            template_set.profit_loss_statement_template = None
        # Deserialize balance_sheet_statement_template
        balance_sheet_statement_template = validated_data.pop('balanceSheetStatementTemplate')
        if balance_sheet_statement_template:
            if balance_sheet_statement_template.get('id', None):
                template_set.balance_sheet_statement_template = BalanceSheetTemplate.objects.get(id=balance_sheet_statement_template.get('id', None))
            else:
                template_set.balance_sheet_statement_template = template_set.balance_sheet_statement_template_id
        else:
            template_set.balance_sheet_statement_template = None
        # Deserialize monthly_project_summary_template
        monthly_project_summary_template = validated_data.pop('monthlyProjectSummaryTemplate')
        if monthly_project_summary_template:
            if monthly_project_summary_template.get('id', None):
                template_set.monthly_project_summary_template = MonthlyProjectSummaryTemplate.objects.get(id=monthly_project_summary_template.get('id', None))
            else:
                template_set.monthly_project_summary_template = template_set.monthly_project_summary_template_id
        else:
            template_set.monthly_project_summary_template = None
        # Deserialize work_report_template
        work_report_template = validated_data.pop('workReportTemplate')
        if work_report_template:
            if work_report_template.get('id', None):
                template_set.work_report_template = WorkReportTemplate.objects.get(id=work_report_template.get('id', None))
            else:
                template_set.work_report_template = template_set.work_report_template_id
        else:
            template_set.work_report_template = None
        template_set.save()



# -*- coding: utf-8 -*-

import factory
from koalixcrm.crm.models import AgreementType


class StandardAgreementTypeFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = AgreementType

    title = 'This is a test Agreement Type'
    description = "This is a description of such a Agreement Type"
